import os
import math
import requests
import json
import datetime
from pygdpr.models.dpa import DPA
from bs4 import BeautifulSoup
from bs4.dammit import EncodingDetector
from pygdpr.services.filename_from_path_service import filename_from_path_service
from pygdpr.services.pdf_to_text_service import PDFToTextService
from pygdpr.specifications import pdf_file_extension_specification
from pygdpr.models.common.pagination import Pagination
from pygdpr.policies.gdpr_policy import GDPRPolicy
from pygdpr.specifications.should_retain_document_specification import ShouldRetainDocumentSpecification
from urllib.parse import urlparse
import hashlib
import textract
import time
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from pygdpr.policies.webdriver_exec_policy import WebdriverExecPolicy
import urllib3

class Bulgaria(DPA):
    def __init__(self, path=os.curdir):
        country_code='BG'
        super().__init__(country_code, path)

    def update_pagination(self, pagination=None, page_soup=None, driver=None, start_path=''):
        source = {
            'host': 'https://www.cpdp.bg/index.php',
            'start_path': start_path
        }
        host = source['host']
        start_path = source['start_path']
        if pagination is None:
            pagination = Pagination()
            pagination.add_item(host + start_path)
        else:
            pages = page_soup.find('div', class_='pages')
            if pages is not None:
                for page_link in pages.find_all('a'):
                    page_href = page_link.get('href')
                    pagination.add_item('https://www.cpdp.bg' + page_href)
        return pagination

    def get_source(self, page_url=None, driver=None):
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        assert (page_url is not None)
        results_response = None
        try:
            results_response = requests.request('GET', page_url, verify=False)

            results_response.raise_for_status()
        except requests.exceptions.HTTPError as error:
            if to_print:
                print(error)
            pass
        return results_response

    def get_docs(self, existing_docs=[], overwrite=False, to_print=True):
        added_docs = []
        # call all the get_docs_X() functions
        added_docs += self.get_docs_DecJudgeOpinion(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_AnnualReports(existing_docs=[], overwrite=False, to_print=True)
        return added_docs

    def get_docs_DecJudgeOpinion(self, existing_docs=[], overwrite=False, to_print=True):
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        print("\n========================= Bulgaria Decisions, Judgements and Opinions ===========================")
        existed_docs = []
        source = {
            'host': 'https://www.cpdp.bg',
            'start_path': '/index.php?p=rubric&aid=3'
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        page_source = self.get_source(page_url=page_url)
        if page_source is None:
            print("This url is not exist.")
        results_soup = BeautifulSoup(page_source.text, 'html.parser')
        assert results_soup
        center_part = results_soup.find('div', class_='center-part')
        assert center_part
        for li in center_part.find_all('li'):
            time.sleep(1)
            href = li.find('a').get('href')
            document_type = li.find('a').get_text()
            document_host = 'https://www.cpdp.bg/index.php'

            year = ''
            for word in document_type.split():
                if word.isdigit():
                    year = str(word)
            # ignore all the documents before GDPR release
            if year < '2018':
                continue
            #print('document_type: ', document_type)
            #print('year: ', year)

            pagination = self.update_pagination(start_path=href)
            while pagination.has_next():
                page_url = pagination.get_next()
                if to_print:
                    print('\tPage: \t', page_url)
                page_soup = self.get_source(page_url=page_url)
                if page_soup is None:
                    continue
                document_soup = BeautifulSoup(page_soup.text, 'html.parser')
                assert document_soup
                center_part = document_soup.find('div', class_='center-part')
                dpa_folder = self.path
                # type 1 structure: Решения на ВАС
                time.sleep(1)
                if 'Решения на ВАС' in document_type:
                    print("SCA Descisons")
                    for articles in center_part.find_all('a'):
                        document_url = articles.get('href')
                        document_title = articles.get_text()
                        if len(document_title) != 1 and document_url.startswith('https:'):
                            print('document_title: ', document_title)
                            print('\tdocument_href: ', document_url)
                            document_hash = hashlib.md5(document_title.encode()).hexdigest()
                            print('\tdocument_hash: ', document_hash)
                            if document_hash in existed_docs:
                                continue
                            existed_docs.append(document_hash)
                            if document_hash in existing_docs and overwrite == False:
                                if to_print:
                                    print('\tSkipping existing document:\t', document_hash)
                                continue
                            document_folder = dpa_folder + '/' + 'SCA Decisions' + '/' + document_hash

                            try:
                                os.makedirs(document_folder)
                                exec_path = WebdriverExecPolicy().get_system_path()
                                options = webdriver.ChromeOptions()
                                options.add_argument('headless')
                                driver_doc = webdriver.Chrome(options=options, executable_path=exec_path)
                                driver_doc.get(document_url)
                                lotusContent = driver_doc.find_element_by_class_name('lotusContent')
                                body_text = lotusContent.text
                                with open(
                                        document_folder + '/' + self.language_code + '.txt',
                                        'w') as f:
                                    f.write(body_text)
                                with open(document_folder + '/' + 'metadata.json', 'w') as f:
                                    metadata = {
                                        'title': {
                                            self.language_code: document_title
                                        },
                                        'md5': document_hash,
                                        'releaseDate': 'Need to add',
                                        # 'releaseDate': date.strftime('%d/%m/%Y'),
                                        'url': document_url
                                    }
                                    json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                            except FileExistsError:
                                print("\tDirectory path already exists, continue.")

                # type 2 structure: Решения на административни съдилища
                elif 'административни съдилища' in document_type or 'Решения на АССГ' in document_type:
                    print('Courts Decisions')
                    for articles in center_part.find_all('a'):
                        if articles.get('href').endswith('.pdf'):
                            document_title = articles.get_text()
                            print(document_title)
                            document_href = articles.get('href')
                            document_url = host + document_href
                            print('document_title: ', document_title)
                            print('\tdocument_url: ', document_url)
                            document_hash = hashlib.md5(document_title.encode()).hexdigest()
                            if document_hash in existed_docs:
                                continue
                            print('\tdocument_hash: ', document_hash)
                            existed_docs.append(document_hash)
                            if document_hash in existing_docs and overwrite == False:
                                if to_print:
                                    print('\tSkipping existing document:\t', document_hash)
                                continue

                            document_folder = dpa_folder + '/' + 'Courts Decisions' + '/' + document_hash
                            try:
                                os.makedirs(document_folder)
                                file_response = None
                                try:
                                    file_response = requests.request('GET', document_url, verify=False)
                                    file_response.raise_for_status()
                                except requests.exceptions.HTTPError as error:
                                    if to_print:
                                        print(error)
                                    pass
                                if file_response is None:
                                    continue
                                if len(file_response.text) == 0:
                                    continue
                                file_content = file_response.content
                                if file_content is None:
                                    continue
                                with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                                    f.write(file_content)
                                with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                                    document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                                    f.write(document_text)
                                with open(document_folder + '/' + 'metadata.json', 'w') as f:
                                    metadata = {
                                        'title': {
                                            self.language_code: document_title
                                        },
                                        'md5': document_hash,
                                        'releaseDate': 'Need to add',
                                        # 'releaseDate': date.strftime('%d/%m/%Y'),
                                        'url': document_url
                                    }
                                    json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                            except FileExistsError:
                                print("\tDirectory path already exists, continue.")
                # type 3 Решения на КЗЛД по жалби, сигнали и други искания AND Становища на КЗЛД
                else:
                    print("CPDP Decisions or Opinion")
                    for div in center_part.find_all('div', class_='news-home'):
                        news_content = div.find('div', class_='news-content')
                        assert news_content
                        h6 = news_content.find('h6')
                        assert h6
                        result_link = h6.find('a')
                        assert result_link
                        document_title = result_link.get_text()
                        print('document title: ', document_title)
                
                        document_hash = hashlib.md5(document_title.encode()).hexdigest()
                        if document_hash in existed_docs:
                            continue
                        existed_docs.append(document_hash)
                        if document_hash in existing_docs and overwrite == False:
                            if to_print:
                                print('\tSkipping existing document:\t', document_hash)
                            continue
                        article_href = result_link.get('href')
                        article_url = document_host + article_href
                        print('\tdocument_url: ', article_url)
                        print('\tdocument_hash: ', document_hash)

                        document_folder = dpa_folder + '/' + 'CPDP Decisions or Opinion' + '/' + document_hash
                        try:
                            os.makedirs(document_folder)
                            file_response = None
                            try:
                                file_response = requests.request('GET', article_url, verify=False )
                                file_response.raise_for_status()
                            except requests.exceptions.HTTPError as error:
                                if to_print:
                                    print(error)
                                pass
                            if file_response is None:
                                continue
                            if len(file_response.text) == 0:
                                continue
                            file_soup = BeautifulSoup(file_response.text, 'html.parser')
                            body_content = file_soup.find('div', class_='center-part')
                            body_text = body_content.get_text()
                           
                            with open(
                                    document_folder + '/' + self.language_code + '.txt', 'w') as f:
                                f.write(body_text)
                            with open(document_folder + '/' + 'metadata.json', 'w') as f:
                                metadata = {
                                    'title': {
                                        self.language_code: document_title
                                    },
                                    'md5': document_hash,
                                    'releaseDate': 'Need to add',
                                    'url': article_url
                                }
                                json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                        except FileExistsError:
                            print("\tDirectory path already exists, continue.")
                print('\n')
                pagination = self.update_pagination(pagination=pagination, page_soup=document_soup, start_path=href)
        return existed_docs

    def get_docs_AnnualReports(self, existing_docs=[], overwrite=False, to_print=True):
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        print("\n========================= Bulgaria Annual Reports ===========================")
        existed_docs = []
        source = {
            'host': 'https://www.cpdp.bg',
            'start_path': '/index.php?p=rubric&aid=14'
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        page_source = self.get_source(page_url=page_url)
        if page_source is None:
            print("This url is not exist.")
        results_soup = BeautifulSoup(page_source.text, 'html.parser')
        assert results_soup
        center_part = results_soup.find('div', class_='center-part')
        assert center_part
        for ul in center_part.find_all('ul'):
            time.sleep(1)
            href = ul.find('a').get('href')
            document_title = ul.find('a').get_text()
            document_hash = hashlib.md5(document_title.encode()).hexdigest()
            host_2 = 'https://www.cpdp.bg/index.php'
            document_url = host_2 + href
            year = document_title.split(' ')[-2]
            if year < '2018':
                break
            print('document_title :', document_title)
            print('\tdocument_url: ', document_url)
            #print('year: ', year)
            page_soup = self.get_source(page_url=document_url)
            if page_soup is None:
                continue
            document_soup = BeautifulSoup(page_soup.text, 'html.parser')
            assert document_soup
            center_part = document_soup.find('div', class_='center-part')
            for a in center_part.find_all('a'):
                if 'PDF' in a.get_text():
                    article_href = a.get('href')
                    article_title = a.get_text()
                    #print('\tarticle_href: ', article_href)
                    print('\tarticle_title: ', article_title)
                    if article_href.startswith('download'):
                        file_url = host + '/' + article_href
                        print('\tfile_url: ', file_url)
                    file_response = None
                    try:
                        file_response = requests.request('GET', file_url, verify=False)
                        file_response.raise_for_status()
                    except requests.exceptions.HTTPError as error:
                        if to_print:
                            print(error)
                        pass
                    if file_response is None:
                        continue
                    if len(file_response.text) == 0:
                        continue
                    file_content = file_response.content
                    if file_content is None:
                        continue
                    dpa_folder = self.path

                    document_folder = dpa_folder + '/' + 'Annual Reports' + '/' + document_hash
                    try:
                        os.makedirs(document_folder)
                        with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                            f.write(file_content)
                        with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                            document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                            f.write(document_text)
                        with open(document_folder + '/' + 'metadata.json', 'w') as f:
                            metadata = {
                                'title': {
                                    self.language_code: document_title
                                },
                                'md5': document_hash,
                                'releaseDate': year,
                                'url': document_url
                            }
                            json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                        print('\n')
                        existed_docs.append(document_hash)
                    except FileExistsError:
                        print("Directory path already exists, continue.")
        return existed_docs









from pygdpr.services.metadata_service import MetadataService
import nltk
from collections import Counter
import heapq

class SummaryMetadataService(MetadataService):
    def for_text(self, text, num_sents=2):
        if len(text) == 0:
            return None
        sentence_list = nltk.sent_tokenize(text)
        stopwords = set(nltk.corpus.stopwords.words('english'))
        words = nltk.word_tokenize(text)
        # words = [w for w in words if w.isalpha()]
        words = [w for w in words if w not in stopwords]
        word_frequencies = dict(Counter(words))
        maximum_frequncy = max(word_frequencies.values())
        for word in word_frequencies.keys():
            word_frequencies[word] = (word_frequencies[word]/maximum_frequncy)
        sentence_scores = {}
        for sent in sentence_list:
            for word in nltk.word_tokenize(sent.lower()):
                if word in word_frequencies.keys():
                    if sent not in sentence_scores.keys():
                        sentence_scores[sent] = word_frequencies[word]
                    else:
                        sentence_scores[sent] += word_frequencies[word]
        summary_sentences = heapq.nlargest(num_sents, sentence_scores, key=sentence_scores.get)
        summary = ' '.join(summary_sentences)
        language_code = 'en'
        return {
            language_code: summary
        }



class PriceTerminateTranslateSpecification():
    def __init__(self, price_terminate_usd):
        self.price_terminate_usd = price_terminate_usd
    def is_satisfied_by(self, price):
        return (self.price_terminate_usd > 0.0 and price >= self.price_terminate_usd)



import os
import time
import math
import requests
import json
import datetime
import hashlib
import dateparser
import re
import csv
from pygdpr.models.dpa import DPA
from bs4 import BeautifulSoup
from pygdpr.services.filename_from_path_service import filename_from_path_service
from pygdpr.services.pdf_to_text_service import PDFToTextService
from pygdpr.specifications import pdf_file_extension_specification
from pygdpr.specifications.should_retain_document_specification import ShouldRetainDocumentSpecification
from pygdpr.models.common.pagination import Pagination
from pygdpr.policies.gdpr_policy import GDPRPolicy
import textract
import sys


class Spain(DPA):
    def __init__(self, path=os.curdir):
        country_code='es'
        super().__init__(country_code, path)

    # Added two new arguments: new_page_type and start_path
    # new_page_type: indicates the type of document we are scraping. This is needed to index the correct link
    # sub-string in source{} when adding additional pages to the pagination object
    # start_path: this is the usual start_path parameter normally contained in source{}. It's just passed as an
    # argument now
    def update_pagination(self, pagination=None, page_soup=None, driver=None, new_page_type=None, start_path=None):
        source = {
            "host": "https://www.aepd.es",
            "new_page_decisions": "/es/informes-y-resoluciones/resoluciones",
            "new_page_reports": "/es/informes-y-resoluciones/informes-juridicos",
            "new_page_guides": "/es/guias-y-herramientas/guias",
            "new_page_infographics": "/es/guias-y-herramientas/infografias",
            "new_page_blogs": "/es/prensa-y-comunicacion/blog"
        }
        host = source['host']
        start_path_new_page = source[new_page_type]

        if pagination is None:
            pagination = Pagination()
            pagination.add_item(host + start_path)
        else:
            assert page_soup is not None
            pager = page_soup.find('nav', class_='pager')
            assert pager
            pager_items = pager.find('ul', class_='pager__items')
            assert pager_items

            li = pager_items.find('li', class_='pager__item--next')
            if li is not None:
                page_link = li.find('a')
                page_href = page_link.get('href')
                page_link = host + start_path_new_page + page_href
                if not pagination.has_link(page_link):
                    print('\n')
                    # print('Adding page: ' + page_link + ' to pagination object')
                    pagination.add_item(page_link)

        return pagination

    def get_source(self, page_url=None, driver=None):
        assert (page_url is not None)
        page_source = None
        try:
            page_source = requests.request('GET', page_url)
            page_source.raise_for_status()
        except requests.exceptions.HTTPError as error:
            pass
        return page_source

    def get_docs(self, existing_docs=[], overwrite=False, to_print=True):
        added_docs = []
        # call all the get_docs_X() functions
        added_docs += self.get_docs_Decisions(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Reports(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Guides(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Infographics(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Blogs(existing_docs=[], overwrite=False, to_print=True)

        return added_docs


    def get_docs_Decisions(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== Spain Decision Documents =========================")

        added_docs = []
        pagination = self.update_pagination(new_page_type='new_page_decisions', start_path='/es/informes-y-resoluciones/resoluciones?f%5B0%5D=ley_tipificacion_de_la_gravedad%3AReglamento%20General%20de%20Protecci%C3%B3n%20de%20Datos')

        iteration = 1
        while pagination.has_next():
            page_url = pagination.get_next()
            if to_print:
                print('\nNEW PAGE: ' + page_url)

            page_source = self.get_source(page_url=page_url)
            page_soup = BeautifulSoup(page_source.text, 'html.parser')
            assert page_soup

            view_content = page_soup.find('div', class_='view-content')
            assert view_content

            for views_row in view_content.find_all('div', class_='views-row'):
                time.sleep(2)
                views_field_title = views_row.find('div', class_='views-field-title')
                assert views_field_title
                result_link = views_field_title.find('a')
                if result_link is None:
                    continue

                document_title = result_link.get_text()

                print('\n------------ Document ' + str(iteration) + ' ------------')
                iteration += 1

                print('\tDocument Title: ' + document_title)

                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite is False:
                    if to_print:
                        print('\tSkipping existing document:', document_hash)
                    continue
                document_href = result_link.get('href')
                assert document_href
                if document_href.endswith('.pdf') is False:
                    continue
                host = "https://www.aepd.es"
                document_url = host + document_href

                print('\tDocument URL: ' + document_url)
                views_field_field_advertise_on = views_row.find('div', class_='views-field-field-advertise-on')
                assert views_field_field_advertise_on
                time_ = views_field_field_advertise_on.find('time')
                assert time_
                date_part = time_.get('datetime')
                date_str = date_part.split('T')[0]
                tmp = datetime.datetime.strptime(date_str, '%Y-%m-%d')
                date = datetime.date(tmp.year, tmp.month, tmp.day)

                print('\tDocument Date: ' + str(date))

                document_year = str(date)[0:4]

                # By the time we reach document from 2017 and below, it is unlikely to encounter any relevant ones
                if int(document_year) < 2018:
                    # sys.exit('Remaining documents outdated')
                    print('Remaining documents outdated')
                    return added_docs

                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    print('Before GDPR adopted')
                    continue

                if to_print:
                    print("\tDocument:", document_hash)

                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue

                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Decisions' + '/' + document_hash

                try:
                    os.makedirs(document_folder)
                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        try:
                            document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                            f.write(document_text)
                        except:
                            print('Failed to convert PDF to text')
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    added_docs.append(document_hash)
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
            pagination = self.update_pagination(pagination=pagination, page_soup=page_soup, new_page_type='new_page_decisions', start_path='/es/informes-y-resoluciones/resoluciones?f%5B0%5D=ley_tipificacion_de_la_gravedad%3AReglamento%20General%20de%20Protecci%C3%B3n%20de%20Datos')
        return added_docs

    def get_docs_Reports(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== Spain Reports =========================")

        added_docs = []
        pagination = self.update_pagination(new_page_type="new_page_reports", start_path='/informes-y-resoluciones/informes-juridicos?f%5B0%5D=informes_disposiciones_estudiadas%3AReglamento%20General%20de%20Protecci%C3%B3n%20de%20Datos%20UE%202016/679')

        iteration = 1
        while pagination.has_next():
            page_url = pagination.get_next()
            if to_print:
                print('\nNEW PAGE: ' + page_url)

            page_source = self.get_source(page_url=page_url)
            page_soup = BeautifulSoup(page_source.text, 'html.parser')
            assert page_soup

            view_content = page_soup.find('div', class_='view-content')
            assert view_content

            for views_row in view_content.find_all('div', class_='views-row'):
                time.sleep(2)
                views_field_title = views_row.find('div', class_='views-field-title')
                assert views_field_title
                result_link = views_field_title.find('a')
                if result_link is None:
                    continue

                document_title = result_link.get_text()

                print('\n------------ Document ' + str(iteration) + ' ------------')
                iteration += 1

                print('\tDocument Title: ' + document_title)

                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite is False:
                    if to_print:
                        print('\tSkipping existing document:', document_hash)
                    continue
                document_href = result_link.get('href')
                assert document_href
                if document_href.endswith('.pdf') is False:
                    continue
                host = "https://www.aepd.es"
                document_url = host + document_href

                print('\tDocument URL: ' + document_url)
                views_field_field_advertise_on = views_row.find('div', class_='views-field-field-advertise-on')
                assert views_field_field_advertise_on
                time_ = views_field_field_advertise_on.find('time')
                assert time_
                date_str = time_.get('datetime')
                date_str = date_str.split('T')[0]
                tmp = datetime.datetime.strptime(date_str, '%Y-%m-%d')
                date = datetime.date(tmp.year, tmp.month, tmp.day)

                print('\tDocument Date: ' + str(date))

                document_year = str(date)[0:4]
                # By the time we reach document from 2017 and below, it is unlikely to encounter any relevant ones
                if int(document_year) < 2018:
                    # sys.exit('Remaining documents outdated')
                    print('Remaining documents outdated')
                    return added_docs

                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    print('Before GDPR adopted')
                    continue

                if to_print:
                    print("\tDocument:", document_hash)

                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue

                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Reports' + '/' + document_hash

                try:
                    os.makedirs(document_folder)

                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        try:
                            document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                            f.write(document_text)
                        except:
                            print('Failed to convert PDF to text')
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    added_docs.append(document_hash)
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
            pagination = self.update_pagination(pagination=pagination, page_soup=page_soup, new_page_type="new_page_reports", start_path='/informes-y-resoluciones/informes-juridicos?f%5B0%5D=informes_disposiciones_estudiadas%3AReglamento%20General%20de%20Protecci%C3%B3n%20de%20Datos%20UE%202016/679')
        return added_docs

    def get_docs_Guides(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== Spain Guides =========================")

        added_docs = []
        pagination = self.update_pagination(new_page_type="new_page_guides", start_path='/es/guias-y-herramientas/guias')

        iteration = 1
        while pagination.has_next():
            page_url = pagination.get_next()
            if to_print:
                print('\nNEW PAGE: ' + page_url)

            page_source = self.get_source(page_url=page_url)
            page_soup = BeautifulSoup(page_source.text, 'html.parser')
            assert page_soup

            view_content = page_soup.find('div', class_='view-content')
            assert view_content

            for views_row in view_content.find_all('div', class_='views-row'):
                time.sleep(2)
                views_field_title = views_row.find('div', class_='views-field-title')
                assert views_field_title
                document_title = views_field_title.get_text()

                views_field_advertise = views_row.find('div', class_='views-field-field-advertise-on')
                assert views_field_advertise

                views_time = views_field_advertise.find('time')
                assert views_time
                document_date = views_time.get('datetime')
                date_str = document_date.split('T')[0]
                tmp = datetime.datetime.strptime(date_str, '%Y-%m-%d')
                date = datetime.date(tmp.year, tmp.month, tmp.day)

                result_link = views_row.find('a')
                if result_link is None:
                    continue

                # document_title = result_link.get_text()

                print('\n------------ Document ' + str(iteration) + ' ------------')
                iteration += 1

                print('\tDocument Title: ' + document_title)

                print('\tDocument Date: ' + date_str)

                document_year = str(date)[0:4]
                # By the time we reach document from 2017 and below, it is unlikely to encounter any relevant ones
                if int(document_year) < 2018:
                    # sys.exit('Remaining documents outdated')
                    print('Remaining documents outdated')
                    return added_docs

                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    print('Before GDPR adopted')
                    continue

                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite is False:
                    if to_print:
                        print('\tSkipping existing document:', document_hash)
                    continue
                document_href = result_link.get('href')
                assert document_href

                host = "https://www.aepd.es"

                if document_href.startswith('http'):
                    document_url = document_href
                else:
                    document_url = host + document_href

                print('\tDocument URL: ' + document_url)

                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue

                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Guides' + '/' + document_hash

                try:
                    os.makedirs(document_folder)

                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        try:
                            document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                            f.write(document_text)
                        except:
                            print('\tFailed to convert PDF to text')
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    added_docs.append(document_hash)
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
            pagination = self.update_pagination(pagination=pagination, page_soup=page_soup, new_page_type="new_page_guides", start_path='/es/guias-y-herramientas/guias')
        return added_docs

    def get_docs_Infographics(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== Spain Infographics =========================")

        added_docs = []
        pagination = self.update_pagination(new_page_type="new_page_infographics", start_path='/es/guias-y-herramientas/infografias')

        iteration = 1
        while pagination.has_next():
            page_url = pagination.get_next()
            if to_print:
                print('\nNEW PAGE: ' + page_url)

            page_source = self.get_source(page_url=page_url)
            page_soup = BeautifulSoup(page_source.text, 'html.parser')
            assert page_soup

            view_content = page_soup.find('div', class_='view-content')
            assert view_content

            for views_row in view_content.find_all('div', class_='views-row'):
                time.sleep(2)
                views_field_title = views_row.find('div', class_='views-field-title')
                assert views_field_title

                # Find the document date
                views_field_advertise = views_row.find('div', class_='views-field-field-advertise-on')
                assert views_field_advertise

                views_time = views_field_advertise.find('time')
                assert views_time
                document_date = views_time.get('datetime')
                date_str = document_date.split('T')[0]
                tmp = datetime.datetime.strptime(date_str, '%Y-%m-%d')
                date = datetime.date(tmp.year, tmp.month, tmp.day)

                # Get the document result link
                result_link = views_field_title.find('a')
                if result_link is None:
                    continue

                document_title = result_link.get_text()

                print('\n------------ Document ' + str(iteration) + ' ------------')
                iteration += 1

                print('\tDocument Title: ' + document_title)

                print('\tDocument Date: ' + date_str)

                document_year = str(date)[0:4]
                # By the time we reach document from 2017 and below, it is unlikely to encounter any relevant ones
                if int(document_year) < 2018:
                    # sys.exit('Remaining documents outdated')
                    print('Remaining documents outdated')
                    return added_docs

                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    print('Before GDPR adopted')
                    continue

                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite is False:
                    if to_print:
                        print('\tSkipping existing document:', document_hash)
                    continue
                document_href = result_link.get('href')
                assert document_href

                host = "https://www.aepd.es"

                if document_href.startswith('http'):
                    document_url = document_href
                else:
                    document_url = host + document_href

                print('\tDocument URL: ' + document_url)

                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue

                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Infographics' + '/' + document_hash

                try:
                    os.makedirs(document_folder)

                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        try:
                            document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                            f.write(document_text)
                        except:
                            print('\tFailed to convert PDF to text')
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': document_date,
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    added_docs.append(document_hash)
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
            pagination = self.update_pagination(pagination=pagination, page_soup=page_soup, new_page_type="new_page_infographics", start_path='/es/guias-y-herramientas/infografias')
        return added_docs

    def get_docs_Blogs(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== Spain Blogs =========================")

        added_docs = []
        pagination = self.update_pagination(new_page_type="new_page_blogs", start_path='/es/prensa-y-comunicacion/blog')

        iteration = 1
        while pagination.has_next():
            page_url = pagination.get_next()
            if to_print:
                print('\nNEW PAGE: ' + page_url)

            page_source = self.get_source(page_url=page_url)
            page_soup = BeautifulSoup(page_source.text, 'html.parser')
            assert page_soup

            view_content = page_soup.find('div', class_='view-content')
            assert view_content

            for li in view_content.find_all('li'):
                time.sleep(2)

                views_field_title = li.find('div', class_='views-field-title')
                assert views_field_title

                # Find the document date
                views_field_advertise = li.find('div', class_='views-field-field-advertise-on')
                assert views_field_advertise

                views_time = views_field_advertise.find('time')
                assert views_time
                document_date = views_time.get('datetime')
                date_str = document_date.split('T')[0]
                tmp = datetime.datetime.strptime(date_str, '%Y-%m-%d')
                date = datetime.date(tmp.year, tmp.month, tmp.day)

                # Get the document result link
                result_link = views_field_title.find('a')
                if result_link is None:
                    continue

                document_title = result_link.get_text()

                print('\n------------ Document ' + str(iteration) + ' ------------')
                iteration += 1

                print('\tDocument Title: ' + document_title)

                print('\tDocument Date: ' + date_str)

                document_year = str(date)[0:4]
                # By the time we reach document from 2017 and below, it is unlikely to encounter any relevant ones
                if int(document_year) < 2018:
                    # sys.exit('Remaining documents outdated')
                    print('Remaining documents outdated')
                    return added_docs

                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    print('Before GDPR adopted')
                    continue

                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite is False:
                    if to_print:
                        print('\tSkipping existing document:', document_hash)
                    continue
                document_href = result_link.get('href')
                assert document_href

                host = "https://www.aepd.es"

                if document_href.startswith('http'):
                    document_url = document_href
                else:
                    document_url = host + document_href

                print('\tDocument URL: ' + document_url)

                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue

                # Get the document text
                document_soup = BeautifulSoup(document_response.text, 'html.parser')
                assert document_soup

                main_content = document_soup.find('main')
                assert main_content

                layout_wrapper = main_content.find('div', class_='layout-wrapper')
                assert layout_wrapper

                document_text = layout_wrapper.get_text()

                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Blogs' + '/' + document_hash

                try:
                    os.makedirs(document_folder)

                    with open(document_folder + '/' + self.language_code + '.txt', 'w') as f:
                        f.write(document_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': document_date,
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    added_docs.append(document_hash)
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
            pagination = self.update_pagination(pagination=pagination, page_soup=page_soup, new_page_type="new_page_blogs", start_path='/es/prensa-y-comunicacion/blog')
        return added_docs



from pygdpr.models.dpa.ireland import *
from pygdpr.models.dpa.united_kingdom import *
from pygdpr.models.dpa.austria import *
from pygdpr.models.dpa.belgium import *
from pygdpr.models.dpa.bulgaria import *
from pygdpr.models.dpa.czech_republic import *
from pygdpr.models.dpa.croatia import *
from pygdpr.models.dpa.cyprus import *
from pygdpr.models.dpa.denmark import *
from pygdpr.models.dpa.estonia import *
from pygdpr.models.dpa.france import *
from pygdpr.models.dpa.germany import *
from pygdpr.models.dpa.hungary import *
from pygdpr.models.dpa.latvia import *
from pygdpr.models.dpa.greece import *
from pygdpr.models.dpa.finland import *
from pygdpr.models.dpa.italy import *
from pygdpr.models.dpa.luxembourg import *
from pygdpr.models.dpa.malta import *
from pygdpr.models.dpa.portugal import *
from pygdpr.models.dpa.slovenia import *
from pygdpr.models.dpa.spain import*
from pygdpr.models.dpa.lithuania import *
from pygdpr.models.dpa.netherlands import *
from pygdpr.models.dpa.poland import *
from pygdpr.models.dpa.edpb import *
from pygdpr.models.dpa.romania import *
from pygdpr.models.dpa.slovakia import *
from pygdpr.models.dpa.sweden import *
import click


print("\n")
print("  ________________ ____________________        .__        _________                      .__")
print(" /  _____/\______ \\\\______   \______   \___  __|__|__  __ \_   ___ \____________ __  _  _|  |   ___________")
print("/   \  ___ |    |  \|     ___/|       _/\  \/  /  \  \/ / /    \  \/\_  __ \__  \\\\ \/ \/ /  | _/ __ \_  __ \\")
print("\    \_\  \|    `   \    |    |    |   \ >    <|  |\   /  \     \____|  | \// __ \\\\     /|  |_\  ___/|  | \/")
print(" \______  /_______  /____|    |____|_  //__/\_ \__| \_/    \______  /|__|  (____  /\/\_/ |____/\___  >__|")
print("        \/        \/                 \/       \/                  \/            \/                 \/")
print("\n")

@click.group()

def cli():
    pass


@cli.command()
@click.option('--country', default='EDPB', help='The country to obtain document from.')
@click.option('--document_type', default='Docs', help='The type of documents to include.')
@click.option('--path', default=None, help='File path where scraped documents should be stored.')
@click.option('--overwrite', default=False, help='Set to True if scraper should overwrite existing documents.')


def scrape(country, document_type, path, overwrite):

    """
        \b
        SUPPORTED COUNTRIES:     DOCUMENTS TYPES:

        \b
        Austria                  Decisions
        Belgium                  Decisions, Annual Reports, Opinions, Guides
        Bulgaria                 Decisions, Annual Reports, Opinions, Judgements
        Croatia                  Decisions, Annual Reports
        Cyprus                   Decisions, Annual Reports
        Czech Republic           Decisions, Annual Reports, Completed Inspections, Court Rulings, Opinions, Press Releases
        Denmark                  Decisions, Annual Reports, Permissions
        EDPB (Agency)            Decisions, Annual Reports, Guidelines, Letters, Opinions, Recommendations
        Estonia                  Instructions, Prescriptions, Annual Reports,
        Finland                  Docs (Contains Advice, Decisions, Guides, Notices)
        France                   Decisions, Notices, Guidelines, Reports
        Germany                  Docs (Contains documents from all the Germany States and Federal DPA)
        Greece                   Decisions, Annual Reports, Guidelines, Opinions, Recommendations
        Hungary                  Decisions, Annual Reports, Notices, Recommendations, Resolutions
        Ireland                  Decisions, Judgements, News, Reports, Blogs, Guidances
        Italy                    Injunctions, Annual Reports, Hearings, Interviews, Newsletters, Publications
        Luxembourg               Annual Reports, Opinions
        Malta                    Guidelines, News Articles
        Netherlands              Decisions, Opinions, Public Disclosures, Reports
        Poland                   Decisions, News
        Portugal                 Decisions, Guidelines, Reports
        Romania                  Docs (Contains Decisions, Reports)
        Slovakia                 Fines, Opinions, Reports
        Slovenia                 Guidelines, Opinions, Reports
        Spain                    Blogs, Decisions, Guides, Infographics, Reports
        Sweden                   Decisions, Guidances, Publications
        United Kingdom           Enforcements, Notices, Reports
    """

    # Path is where the user stores documents, something like: '/Users/evanjacobs/Desktop/test_scraper'
    # If no path is give, exit
    if path is None:
        sys.exit("No file path given.")

    try:
        os.makedirs(path)
    except FileExistsError:
        pass

    # Create visitedDocs.txt if it doesn't already exist
    try:
        hashFile = open(path + "/visitedDocs.txt", "x")
        hashFile.close()
    except FileExistsError:
        print('visitedDocs already exists')
        pass

    existing_docs = []
    # Open visitedDocs.txt and read the contents into existing_docs list
    # Change this to use .readlines() and get contents all at once into list
    with open(path + "/visitedDocs.txt") as file:
        for line in file:
            hash_n_stripped = line.rstrip('\n')
            # Add the hash's in visitedDocs.txt to existing_docs
            existing_docs.append(hash_n_stripped)

    # Open the file again (with 'append'), since it is closed after previous looping
    hashFile = open(path + "/visitedDocs.txt", "a")

    # Determine country, path, and instantiate the DPA
    if country == "Austria":
        path = path + "/austria"
        dpa = Austria(path=path)
    elif country == "Belgium":
        path = path + "/belgium"
        dpa = Belgium(path=path)
    elif country == "Bulgaria":
        path = path + "/bulgaria"
        dpa = Bulgaria(path=path)
    elif country == "Croatia":
        path = path + "/croatia"
        dpa = Croatia(path=path)
    elif country == "Cyprus":
        path = path + "/cyrpus"
        dpa = Cyprus(path=path)
    elif country == "Czech Republic":
        path = path + "/czech_republic"
        dpa = CzechRepublic(path=path)
    elif country == "Denmark":
        path = path +  "/denmark"
        dpa = Denmark(path=path)
    elif country == "EDPB":
        path = path + "/edpb"
        dpa = EDPB(path=path)
    elif country == "Estonia":
        path = path + "/estonia"
        dpa = Estonia(path=path)
    elif country == "Finland":
        path = path + "/finland"
        dpa = Finland(path=path)
    elif country == "France":
        path = path + "/france"
        dpa = France(path=path)
    elif country == "Germany":
        path = path + "/germany"
        dpa = Germany(path=path)
    elif country == "Greece":
        path = path + "/greece"
        dpa = Greece(path=path)
    elif country == "Hungary":
        path = path +  "/hungary"
        dpa = Hungary(path=path)
    elif country == "Ireland":
        path = path + "ireland"
        dpa = Ireland(path=path)
    elif country == "Italy":
        path = path + "/italy"
        dpa = Italy(path=path)
    elif country == "Luxembourg":
        path = path + "/luxembourg"
        dpa = Luxembourg(path=path)
    elif country == "Malta":
        path = path + "/malta"
        dpa = Malta(path=path)
    elif country == "Netherlands":
        path = path + "/netherlands"
        dpa = Netherlands(path=path)
    elif country == "Poland":
        path = path +  "/poland"
        dpa = Poland(path=path)
    elif country == "Portugal":
        path = path + "/portugal"
        dpa = Portugal(path=path)
    elif country == "Romania":
        path = path + "/romania"
        dpa = Romania(path=path)
    elif country == "Slovakia":
        path = path + "/slovakia"
        dpa = Slovakia(path=path)
    elif country == "Slovenia":
        path = path + "/slovenia"
        dpa = Slovenia(path=path)
    elif country == "Spain":
        path = path + "/spain"
        dpa = Spain(path=path)
    elif country == "Sweden":
        path = path + "/sweden"
        dpa = Sweden(path=path)
    else:
        path = path + "/united_kingdom"
        dpa = UnitedKingdom(path=path)

    # existing_docs contains document hashes from visitedDocs.txt
    # added_docs contains hashes of freshly downloaded documents

    # Call appropriate method for desired document type (Note that these are all encompassing, but DPA type will
    # differentiate)
    if document_type == "Decisions":
        if country == "Czech Republic":
            added_docs = dpa.get_docs_Decisions(existing_docs=existing_docs, overwrite=overwrite)
            added_docs += dpa.get_docs_DecisionOfPresident(existing_docs=[], overwrite=False, to_print=True)
            added_docs += dpa.get_docs_DecisionMakingActivites(existing_docs=[], overwrite=False, to_print=True)
        elif country == "Bulgaria":
            added_docs = dpa.get_docs_DecJudgeOpinion(existing_docs=[], overwrite=False, to_print=True)
        elif country == "Sweden":
            added_docs = dpa.get_docs_decisionsAndJudgements(existing_docs=[], overwrite=False, to_print=True)
        else:
            added_docs = dpa.get_docs_Decisions(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Guidelines":
        added_docs = dpa.get_docs_Guidelines(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Annual Reports":
        added_docs = dpa.get_docs_AnnualReports(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Recommendations":
        added_docs = dpa.get_docs_Recommendations(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Opinions":
        added_docs = dpa.get_docs_Opinions(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Letters":
        added_docs = dpa.get_docs_Letters(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Guides":
        added_docs = dpa.get_docs_Guides(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Permissions":
        added_docs = dpa.get_docs_Permissions(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Instructions":
        added_docs = dpa.get_docs_Instructions(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Prescriptions":
        added_docs = dpa.get_docs_Prescriptions(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Notices":
        added_docs = dpa.get_docs_Notices(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Resolutions":
        added_docs = dpa.get_docs_Resolutions(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Interviews":
        added_docs = dpa.get_docs_Interviews(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Publications":
        added_docs = dpa.get_docs_Publications(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Newsletters":
        added_docs = dpa.get_docs_Newsletters(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Injunctions":
        added_docs = dpa.get_docs_Injunctions(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Hearings":
        added_docs = dpa.get_docs_Hearings(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Judgements":
        added_docs = dpa.get_docs_Judgements(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Reports":
        added_docs = dpa.get_docs_Reports(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Enforcements":
        added_docs = dpa.get_docs_Enforcements(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Guidances":
        added_docs = dpa.get_docs_Guidances(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Blogs":
        added_docs = dpa.get_docs_Blogs(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Infographics":
        added_docs = dpa.get_docs_Infographics(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Tutorials":
        added_docs = dpa.get_docs_Tutorials(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Public Disclosures":
        added_docs = dpa.get_docs_publicDisclosure(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "News Articles":
        added_docs = dpa.get_docs_NewsArticles(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Inspection Reports":
        added_docs = dpa.get_docs_InspectionReports(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Violations":
        added_docs = dpa.get_docs_Violations(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "News":
        added_docs = dpa.get_docs_News(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Press Releases":
        added_docs = dpa.get_docs_PressReleases(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Court Rulings":
        added_docs = dpa.get_docs_CourtRulings(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Completed Inspections":
        added_docs = dpa.get_docs_CompletedInspections(existing_docs=existing_docs, overwrite=overwrite)
    elif document_type == "Docs":
        added_docs = dpa.get_docs(existing_docs=existing_docs, overwrite=overwrite)
    else:
        sys.exit("'document_type' argument doesn't match an existing document type. Use --help for more info.")
        pass

    # Iterate through added_docs and add hashes to visitedDocs.txt
    for document_hash in added_docs:
        # Document_hash isn't in visitedDocs.txt -> add it
        document_hash_newline = document_hash + "\n"
        hashFile.write(document_hash_newline)

    # Lets us see written contents during run time
    hashFile.flush()
    # Close visited docs when done scraping
    hashFile.close()

if __name__ == '__main__':
    scrape()


import os
import requests
import json

class GithubCommitService():
    def get_auth(self):
        username, password = os.environ['gh-username'], os.environ['gh-password']
        return username, password

    def get_repo(self):
        owner, name = os.environ['gh-owner'], os.environ['gh-name']
        return owner, name

    def latest_commit(self, params={}):
        owner, name = self.get_repo()
        url = f"https://api.github.com/repos/{owner}/{name}/commits"
        session = requests.Session()
        username, password = self.get_auth()
        session.auth = (username, password)
        response = session.get(url, params=params)
        status_code = response.status_code
        if status_code != 200:
            raise ValueError('Could not get Github commits')
        commits = response.json()
        latest = commits[0] if len(commits) > 0 else None
        return latest



from pygdpr.services.metadata_service import MetadataService
import nltk

class CitationsCountMetadataService(MetadataService):
    def for_text(self, text):
        count = 0
        targets = ['gdpr', 'rgpd', '2016/679']
        words = nltk.word_tokenize(text)
        words = [w.lower() for w in words]
        for w in words:
            if w in targets:
                count += 1
        return count



import os
import math
import requests
import json
import datetime
import hashlib
import dateparser
import re
import csv
from pygdpr.models.dpa import DPA
from bs4 import BeautifulSoup
from pygdpr.services.filename_from_path_service import filename_from_path_service
from pygdpr.services.pdf_to_text_service import PDFToTextService
from pygdpr.specifications import pdf_file_extension_specification
from pygdpr.specifications.should_retain_document_specification import ShouldRetainDocumentSpecification
from pygdpr.models.common.pagination import Pagination
from pygdpr.policies.gdpr_policy import GDPRPolicy
import textract

class Romania(DPA):
    def __init__(self, path=os.curdir):
        country_code='ro'
        super().__init__(country_code, path)

    def update_pagination(self, pagination=None, page_soup=None, driver=None):
        source = {
            "host": "https://www.dataprotection.ro",
            "start_path": "/?page=allnews&lang=ro"
        }
        host = source['host']
        start_path = source['start_path']
        if pagination is None:
            pagination = Pagination()
            pagination.add_item(host + start_path)
        return pagination

    def get_source(self, page_url=None, driver=None):
        assert (page_url is not None)
        results_response = None
        try:
            results_response = requests.request('GET', page_url)
            results_response.raise_for_status()
        except requests.exceptions.HTTPError as error:
            if to_print:
                print(error)
            pass
        return results_response

    def get_docs(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n===================== Decisions & Reports Decision Documents ====================")
        iteration = 1
        added_docs = []
        dict_hashcode = {}
        pagination = self.update_pagination()
        # s0. Pagination
        while pagination.has_next():
            page_url = pagination.get_next()
            if to_print:
                print('Page:\t', page_url)
            page_source = self.get_source(page_url=page_url)
            if page_source is None:
                continue
            results_soup = BeautifulSoup(page_source.text, 'html.parser')
            assert results_soup
            rectangle_scroll = results_soup.find('div', id='rectangle_scroll')
            assert rectangle_scroll
            date_index, title_index, link_index = 0, 2, 3
            p_all = rectangle_scroll.find_all('p')
            assert len(p_all) > 0
            # s1. Results
            for i in range(1, len(p_all)):
                p_date = p_all[i]
                date_str = p_date.get_text()
                tmp = None
                try:
                    tmp = datetime.datetime.strptime(date_str, '%d/%m/%Y')
                except ValueError:
                    pass
                if tmp is None:
                    continue
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    continue
                # s2. Documents
                print('\n------------ Document ' + str(iteration) + ' ------------')
                iteration += 1

                title = p_all[i + 1]
                document_title = title.get_text().strip()
                print('\tdocument_title:\t', document_title)
                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                result_link = None
                j, j_threshold = 0, 4
                while result_link is None:
                    cand_link = p_all[i + j]
                    result_link = cand_link.find('a')
                    if j == j_threshold:
                        break
                    j += 1
                assert result_link
                document_href = result_link.get('href')
                host = "https://www.dataprotection.ro"
                document_url = host + document_href
                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue
                document_soup = BeautifulSoup(document_response.text, 'html.parser')
                assert document_soup
                document_target_area = document_soup.find('div', id='rectangle_scroll')
                assert document_target_area
                document_text = document_target_area.get_text()
                document_text = document_text.strip()
                dpa_folder = self.path

                # solve documents with same title and same date issue
                if document_hash not in dict_hashcode:
                    dict_hashcode[document_hash] = 1
                else:
                    dict_hashcode[document_hash] = dict_hashcode[document_hash] + 1

                document_hash = document_hash + '-' + str(dict_hashcode[document_hash])
                if to_print:
                    print("\tDocument:\t", document_hash)
                    print("\tdate_str: ", date_str)
                    print("\tdocument_url: ", document_url)
                document_folder = dpa_folder + '/' + "Decisions & Reports" + '/' + document_hash

                try:
                    os.makedirs(document_folder)

                    with open(document_folder + '/' + self.language_code + '.txt', 'w') as f:
                        f.write(document_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    added_docs.append(document_hash)
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
        return added_docs




import os
import json
from urllib.parse import urlparse
from pygdpr.models.common.reachability_node import *

path = os.path.abspath('pygdpr/assets/dpa-reachability-info.json')
f = open(path, 'r')
dpa_reachability_info = json.load(f)
f.close()

class BuildDPANodeService:
    def build_dpa_node(self, dpa):
        key = dpa.country_code
        return DPANode(key, dpa)

    def build_source_node(self, dpa):
        pagination = dpa.update_pagination()
        assert (pagination.has_next())
        item = pagination.get_next()
        page_url = None
        page_source = None
        if type(item) == str:
            page_url = item
            page_source = dpa.get_source(page_url=page_url)
            page_source = page_source.text
        else:
            driver = item
            page_url = driver.current_url
            page_source = dpa.get_source(driver=driver)
        if page_source is None:
            return None
        key = urlparse(page_url).path
        return SourceNode(key, page_source)

    def build_label_node(self, label):
        key = label
        return LabelNode(key, label)

    def build_xpath_node(self, xpath):
        key = '.../' + xpath.split('/')[-1]
        return XPathNode(key, xpath)

    def for_dpa(self, dpa):
        dpa_node = self.build_dpa_node(dpa)
        source_node = self.build_source_node(dpa)
        if source_node is None:
            return None
        reachability_info = dpa_reachability_info[dpa.country_code]
        if reachability_info is None:
            dpa_node.set_color(Color.BLACK)
            return dpa_node
        for label, xpaths in reachability_info.items():
            label_node = self.build_label_node(label)
            for xpath in xpaths:
                xpath_node = self.build_xpath_node(xpath)
                label_node.add_child(xpath_node)
            source_node.add_child(label_node)
        dpa_node.add_child(source_node)
        return dpa_node



from graphviz import Graph
import os

class ExportReachabilityAnalysisService:
    def get_height(self, node):
        h = 0
        while len(node.children) > 0:
            h += 1
            node = node.children[-1]
        return h

    def get_node_color(self, node):
        return node.get_color().value

    def get_node_text_color(self, node):
        text_color = 'black'
        node_color = self.get_node_color(node)
        if node_color == 'black':
            text_color = 'white'
        return text_color

    def get_node_id(self, node):
        node_id = ""
        parent = node.parent
        while parent is not None:
            node_id += parent.key
            parent = parent.parent
        node_id += node.key
        return node_id

    def extend_dot_with_node(self, dot, node):
        node_id = self.get_node_id(node)
        label = node.key
        dot.node(
            node_id,
            label,
            style='filled',
            color=self.get_node_color(node),
            fontcolor=self.get_node_text_color(node)
        )
        return dot

    def extend_dot_with_edge(self, dot, node_a, node_b):
        node_a_id = self.get_node_id(node_a)
        node_b_id = self.get_node_id(node_b)
        dot.edge(node_a_id, node_b_id, constraint='true')
        return dot

    def build_dot(self, node, depth, dot=None):
        assert depth >= 0
        if dot is None:
            dot = Graph(format='svg')
            self.extend_dot_with_node(dot, node)
        if depth == 0:
            return dot
        children = node.get_children()
        for i in range(len(children)):
            child_node = children[i]
            self.extend_dot_with_node(dot, child_node)
            self.extend_dot_with_edge(dot, node, child_node)
            if i == len(children) - 1:
                depth -= 1
            self.build_dot(child_node, depth, dot)
        return dot

    def for_node(self, node, path, depth=None):
        max_depth = self.get_height(node)
        if depth is None:
            depth = max_depth
        assert 0 <= depth <= max_depth
        path = os.path.abspath(path)
        dot = self.build_dot(node, depth)
        dot.render(path, view=False)
        return path



import os
import math
import time

import requests
import json
import datetime
import hashlib
import dateparser
import re
from pygdpr.models.dpa import DPA
from bs4 import BeautifulSoup
from pygdpr.services.filename_from_path_service import filename_from_path_service
from pygdpr.services.pdf_to_text_service import PDFToTextService
from pygdpr.specifications import pdf_file_extension_specification
from pygdpr.specifications.should_retain_document_specification import ShouldRetainDocumentSpecification
from pygdpr.models.common.pagination import Pagination
from pygdpr.policies.gdpr_policy import GDPRPolicy
import textract
import selenium
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from pygdpr.policies.webdriver_exec_policy import WebdriverExecPolicy

class Latvia(DPA):
    def __init__(self, path=os.curdir):
        country_code='LV'
        super().__init__(country_code, path)

    def update_pagination(self, pagination=None, page_soup=None, driver=None):
        source = {
            "host": "https://www.dvi.gov.lv",
            "start_path": "/lv/jaunumi"
        }
        host = source['host']
        start_path = source['start_path']
        if pagination is None:
            pagination = Pagination()
            pagination.add_item(host + start_path)
        else:
            ul_pagination = page_soup.find('ul', class_='pagination')
            if ul_pagination is not None:
                for page_item in ul_pagination.find_all('li', class_='page-item'):
                    page_link = page_item.find('a')
                    if page_link is None:
                        continue
                    page_href = page_link.get('href')
                    pagination.add_item(host + start_path + page_href)
        return pagination

    def get_source(self, page_url=None, driver=None):
        assert (page_url is not None)
        results_response = None
        try:
            results_response = requests.request('GET', page_url)
            results_response.raise_for_status()
        except requests.exceptions.HTTPError as error:
            pass
        return results_response

    def get_docs(self, existing_docs=[], overwrite=False, to_print=True):
        added_docs = []
        # call all the get_docs_X() functions
        #added_docs += self.get_docs_Decisions(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Violations(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_AnnualReports(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Opinions(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Guidances(existing_docs=[], overwrite=False, to_print=True)
        return added_docs

    def get_docs_Guidances(self, existing_docs=[], overwrite=False, to_print=True):
        added_docs = []
        added_docs += self.get_docs_guidance_1()
        added_docs += self.get_docs_guidance_2()
        return added_docs

    def get_docs_Decisions(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== Latvia Decision Documents =========================")
        iteration = 1
        existed_docs = []
        source = {
            "host": "https://www.dvi.gov.lv",
            "start_path": "/lv/lemumi"
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        if to_print:
            print('Page:\t', page_url)

        # There is a popup window, use selenium to interact
        exec_path = WebdriverExecPolicy().get_system_path()
        options = webdriver.ChromeOptions()
        options.add_argument('headless')
        driver_doc = webdriver.Chrome(options=options, executable_path=exec_path)
        driver_doc.get(page_url)
        try:
            if driver_doc.find_element_by_class_name('modal-popup-cancel') is not None:
                print("Cancelling pop up window")
                driver_doc.find_element_by_class_name('modal-popup-cancel').click()
                time.sleep(2)
        except selenium.common.exceptions.NoSuchElementException:
            pass

        try:
            # Accept all the cookies before view the web page
            if driver_doc.find_element_by_class_name('cookie-wrapper') is not None:
                print('Accepting all the cookies before view the web page')
                driver_doc.find_element_by_class_name('cookie-accept-all').click()
                time.sleep(2)
        except selenium.common.exceptions.NoSuchElementException:
            pass

        page_source = driver_doc.page_source
        results_soup = BeautifulSoup(page_source, 'html.parser')

        block_content = results_soup.find('div', class_='block-ministry-content')
        node = block_content.find('div', class_='node')
        content = node.find('div', class_='content')
        for accordion in content.find_all('div', class_='accordion'):
            card_year = accordion.find('button', class_='btn btn-link')
            year = card_year.get_text().strip()
            print("year: ", year)
            card_body = accordion.find('div', class_='card-body')
            tr_all = table = card_body.find_all('tr')
            assert tr_all
            tr_all = tr_all[1:] # skip the fst row
            for tr in tr_all:

                print('\n------------ Document ' + str(iteration) + ' ------------')
                iteration += 1

                td_all = tr.find_all('td')
                if len(td_all) == 0:
                    continue
                # year_flag = '2021'
                if year == '2021' or year == '2022':
                    manager_index, remedy_index, pdf_index, date_index = 0, 1, 2, 3
                else:
                    manager_index, pdf_index, date_index, decision_status, court_judgment = 0, 1, 2, 3, 4
                    year_flag = '2020'
                td_date = td_all[date_index]
                assert td_date
                date_str = td_date.get_text().strip()
                if not date_str[0].isdigit():
                    continue
                tmp = datetime.datetime.strptime(date_str, '%d.%m.%Y.')
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    continue
                # no document title shows in table, replace with manager + date
                document_title = td_all[manager_index].get_text().strip() + '-' + date_str
                print('document_title: ', document_title)
                document_hash = hashlib.md5(document_title.encode()).hexdigest()

                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                td_pdf = td_all[pdf_index]
                assert td_pdf
                if td_pdf.find('a') == None:
                    continue
                document_href = td_pdf.find('a').get('href')
                document_url = host + document_href
                print('\tdocument_hash: ', document_hash)
                assert document_url
                print('\tdocument_url" ', document_url)

                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Decisions' + '/' + document_hash


                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue

                document_content = document_response.content

                try:
                    # use selenium to download the pdf, since requests.request can't get the page content
                    os.makedirs(document_folder)
                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        try:
                            document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                            f.write(document_text)
                        except:
                            print('Failed to convert PDF to text')
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs.append(document_hash)
                    '''
                    exec_path = WebdriverExecPolicy().get_system_path()
                    options = webdriver.ChromeOptions()
                    options.add_argument('headless')
                    dpa_folder = self.path
                    document_folder = dpa_folder + '/' + 'Decisions' + '/' + document_hash
                    options.add_experimental_option('prefs', {
                        "download.default_directory": document_folder,  # Change default directory for downloads
                        "download.prompt_for_download": False,  # To auto download the file
                        "download.directory_upgrade": True,
                        "plugins.always_open_pdf_externally": True  # It will not show PDF directly in chrome
                    })
                    driver_doc = webdriver.Chrome(options=options, executable_path=exec_path)
                    driver_doc.get(document_url)
                    time.sleep(5)

                    for root, dirs, files in os.walk(document_folder):
                        # change the pdf file name to "lv.pdf"
                        for name in files:
                            oldname = os.path.join(root, name)
                            print(oldname)
                            newname = os.path.join(root, 'lv.pdf')
                            os.rename(oldname, newname)
                        #with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        #    f.write(document_content)
                        with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                            document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                            f.write(document_text)
                        with open(document_folder + '/' + 'metadata.json', 'w') as f:
                            metadata = {
                                'title': {
                                    self.language_code: document_title
                                },
                                'md5': document_hash,
                                'releaseDate': date.strftime('%d/%m/%Y'),
                                'url': document_url
                            }
                            json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                        existed_docs.append(document_hash)
                    '''
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
        return existed_docs

    def get_docs_Decisions_oldDesign(self, existing_docs=[], overwrite=False, to_print=True):
        existed_docs = []
        source = {
            "host": "https://www.dvi.gov.lv",
            "start_path": "/lv/lemumi"
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        if to_print:
            print('Page:\t', page_url)

        page_source = self.get_source(page_url=page_url)
        results_soup = BeautifulSoup(page_source.text, 'html.parser')
        assert results_soup
        block_content = results_soup.find('div', class_='block-ministry-content')
        node = block_content.find('div', class_='node')
        content = node.find('div', class_='content')
        for accordion in content.find_all('div', class_='accordion'):
            card_year = accordion.find('button', class_='btn btn-link')
            year = card_year.get_text().strip()
            print("year: ", year)
            card_body = accordion.find('div', class_='card-body')
            tr_all = table = card_body.find_all('tr')
            assert tr_all
            tr_all = tr_all[1:] # skip the fst row
            for tr in tr_all:
                td_all = tr.find_all('td')
                if len(td_all) == 0:
                    continue
                if year == '2021' or year == '2022':
                    manager_index, remedy_index, pdf_index, date_index = 0, 1, 2, 3
                else:
                    manager_index, pdf_index, date_index, decision_status, court_judgment = 0, 1, 2, 3, 4
                    year_flag = '2020'
                td_date = td_all[date_index]
                assert td_date
                date_str = td_date.get_text().strip()
                if not date_str[0].isdigit():
                    continue
                tmp = datetime.datetime.strptime(date_str, '%d.%m.%Y.')
                date = datetime.date(tmp.year, tmp.month, tmp.day)
                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    continue
                document_title = td_all[manager_index].get_text().strip() + '-' + date_str
                print('document_title: ', document_title)
                document_hash = hashlib.md5(document_title.encode()).hexdigest()

                if document_hash in existing_docs and overwrite == False:
                    if to_print:
                        print('\tSkipping existing document:\t', document_hash)
                    continue
                td_pdf = td_all[pdf_index]
                assert td_pdf
                if td_pdf.find('a') == None:
                    continue
                document_href = td_pdf.find('a').get('href')
                document_url = host + document_href
                print('\tdocument_hash: ', document_hash)
                assert document_url
                print('\tdocument_url" ', document_url)
                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if document_response is None:
                    continue
                document_content = document_response.content
                dpa_folder = self.path
                document_folder = dpa_folder + '/' + 'Decisions' + '/' + document_hash
                # document_folder = dpa_folder + '/latvia' + '/' + 'Decisions' + '/' + document_hash
                try:
                    os.makedirs(document_folder)

                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                        f.write(document_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                        document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                        f.write(document_text)
                    if year_flag == '2020':
                        judgement = td_all[court_judgment].find('a')
                        if judgement is not None:
                            judgement_href = td_all[court_judgment].find('a').get('href')
                            judgement_url = host + document_href
                            print('\tdocument_2_url: ', judgement_url)
                            judgement_response = None
                            try:
                                judgement_response = requests.request('GET', judgement_url)
                                judgement_response.raise_for_status()
                            except requests.exceptions.HTTPError as error:
                                if to_print:
                                    print(error)
                                pass
                            if judgement_response is None:
                                continue
                            judgement_content = judgement_response.content

                            with open(document_folder + '/' + self.language_code + '_Court Judgement' + '.pdf', 'wb') as f:
                                f.write(judgement_content)
                            with open(document_folder + '/' + self.language_code + '_Court Judgement' + '.txt', 'wb') as f:
                                judgement_text = textract.process(document_folder + '/' + self.language_code + '_Court Judgement' + '.pdf')
                                f.write(judgement_text)
                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                    existed_docs.append(document_hash)
                except FileExistsError:
                    print("Directory path already exists, continue.")
        return existed_docs

    def get_docs_Violations(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== Latvia Violations=========================")
        iteration = 1
        existed_docs = []
        source = {
            "host": "https://www.dvi.gov.lv",
            "start_path": "/lv/saraksts-par-publisko-tiesibu-subjektu-pielautajiem-vdar-prasibu-parkapumiem-un-noversanu"
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        if to_print:
            print('Page:\t', page_url)
        page_source = self.get_source(page_url=page_url)
        results_soup = BeautifulSoup(page_source.text, 'html.parser')
        assert results_soup
        block_content = results_soup.find('div', class_='block-ministry-content')
        node = block_content.find('div', class_='node')
        content = node.find('div', class_='content')
        for accordion in content.find_all('div', class_='accordion'):
            print('\n------------ Document ' + str(iteration) + ' ------------')
            iteration += 1

            card_year = accordion.find('button', class_='btn btn-link')
            year = card_year.get_text().strip().split('.')[0]
            print("\tyear: ", year)
            card_body = accordion.find('div', class_='card-body')
            document_href = card_body.find('a').get('href')
            document_url = host + document_href
            print('\tdocument_url: ', document_url)
            assert document_url

            # no title, so use year to be the title
            document_title = year
            document_hash = hashlib.md5(document_title.encode()).hexdigest()
            print('\tdocument_hash: ', document_hash)
            if document_hash in existing_docs and overwrite == False:
                if to_print:
                    print('\tSkipping existing document:\t', document_hash)
                continue
            document_response = None
            try:
                document_response = requests.request('GET', document_url)
                document_response.raise_for_status()
            except requests.exceptions.HTTPError as error:
                if to_print:
                    print(error)
                pass
            if document_response is None:
                continue
            document_content = document_response.content
            dpa_folder = self.path
            document_folder = dpa_folder + '/' + 'Violations' + '/' + document_hash
            # document_folder = dpa_folder + '/latvia' + '/' + 'Violations' + '/' + document_hash
            try:
                os.makedirs(document_folder)

                with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                    f.write(document_content)
                with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                    document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                    f.write(document_text)
                with open(document_folder + '/' + 'metadata.json', 'w') as f:
                    metadata = {
                        'title': {
                            self.language_code: document_title
                        },
                        'md5': document_hash,
                        'releaseDate': year,
                        'url': document_url
                    }
                    json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                existed_docs.append(document_hash)
            except FileExistsError:
                print("\tDirectory path already exists, continue.")
        return existed_docs


    def get_docs_AnnualReports(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== Latvia Annual Reports=========================")
        iteration = 1
        existed_docs = []
        source = {
            "host": "https://www.dvi.gov.lv",
            "start_path": "/lv/publikacijas-un-parskati"
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        if to_print:
            print('Page:\t', page_url)
        page_source = self.get_source(page_url=page_url)
        results_soup = BeautifulSoup(page_source.text, 'html.parser')
        assert results_soup
        block_content = results_soup.find('div', class_='block-ministry-content')
        node = block_content.find('div', class_='node')
        content = node.find('div', class_='content')
        for paragraph in content.find_all('div', class_='paragraph--type--data'):
            print('\n------------ Document ' + str(iteration) + ' ------------')
            iteration += 1
            year = paragraph.get_text().strip().split('.')[0]
            if year < '2018':
                continue
            print("\tyear: ", year)
            document_href = paragraph.find('a').get('href')
            document_url = host + document_href
            print('\tdocument_url: ', document_url)
            assert document_url
            print('\tdocument_hash: ', document_hash)
            # no title, so use year to be the title
            document_title = year
            document_hash = hashlib.md5(document_title.encode()).hexdigest()

            if document_hash in existing_docs and overwrite == False:
                if to_print:
                    print('\tSkipping existing document:\t', document_hash)
                continue
            document_response = None
            try:
                document_response = requests.request('GET', document_url)
                document_response.raise_for_status()
            except requests.exceptions.HTTPError as error:
                if to_print:
                    print(error)
                pass
            if document_response is None:
                continue
            document_content = document_response.content
            dpa_folder = self.path
            document_folder = dpa_folder + '/' + 'Annual Reports' + '/' + document_hash
            # document_folder = dpa_folder + '/latvia' + '/' + 'Annual Reports' + '/' + document_hash
            try:
                os.makedirs(document_folder)

                with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                    f.write(document_content)
                with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                    document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                    f.write(document_text)
                with open(document_folder + '/' + 'metadata.json', 'w') as f:
                    metadata = {
                        'title': {
                            self.language_code: document_title
                        },
                        'md5': document_hash,
                        'releaseDate': year,
                        'url': document_url
                    }
                    json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                existed_docs.append(document_hash)
            except FileExistsError:
                print("Directory path already exists, continue.")
        return existed_docs

    def get_docs_Opinions(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== Latvia Opinions=========================")
        iteration = 1
        existed_docs = []
        source = {
            "host": "https://www.dvi.gov.lv",
            "start_path": "/lv/dviskaidro"
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        if to_print:
            print('Page:\t', page_url)
        page_source = self.get_source(page_url=page_url)
        results_soup = BeautifulSoup(page_source.text, 'html.parser')
        assert results_soup
        block_content = results_soup.find('div', class_='block-ministry-content')
        node = block_content.find('div', class_='node')
        content = node.find('div', class_='content')
        for paragraph in content.find_all('div', class_='paragraph--view-mode--default'):

            articles_wrapper = paragraph.find('div', class_='articles-wrapper')
            assert articles_wrapper
            article_details = articles_wrapper.find('div', class_='article-details')
            assert article_details
            date_div = article_details.find('div', class_='date')
            assert date_div
            date_str = date_div.get_text().strip()
            print('date_str:', date_str)
            tmp = datetime.datetime.strptime(date_str, '%d.%m.%Y.')
            date = datetime.date(tmp.year, tmp.month, tmp.day)
            if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                continue
            # s1. Results
            article_title = articles_wrapper.find('div', class_='title')
            assert article_title
            a = article_title.find('a')
            # s2. Documents
            print('\n------------ Document ' + str(iteration) + ' ------------')
            iteration += 1

            document_title = a.get_text()
            document_hash = hashlib.md5(document_title.encode()).hexdigest()
            if document_hash in existing_docs and overwrite == False:
                if to_print:
                    print('\tSkipping existing document:\t', document_hash)
                continue
            host = 'https://www.dvi.gov.lv'
            result_link = a.get('href')
            document_href = result_link
            assert document_href
            document_url = host + document_href
            if to_print:
                print("\tdocument_title:\t", document_title)
                print("\tDocument:\t", document_hash)
                print('\tdate:\t', date)
                print('\tdocument_url:\t', document_url)
            document_response = None
            try:
                document_response = requests.request('GET', document_url)
                document_response.raise_for_status()
            except requests.exceptions.HTTPError as error:
                pass
            if document_response is None:
                continue
            document_soup = BeautifulSoup(document_response.text, 'html.parser')
            content_area = document_soup.find('div', id='content-area')
            document_text = content_area.get_text()
            dpa_folder = self.path
            document_folder = dpa_folder + '/' + 'Opinions' + '/' + document_hash
            # document_folder = dpa_folder + '/latvia' + '/' + 'Opinions' + '/' + document_hash
            try:
                os.makedirs(document_folder)

                with open(document_folder + '/' + self.language_code + '.txt', 'w') as f:
                    f.write(document_text)
                with open(document_folder + '/' + 'metadata.json', 'w') as f:
                    metadata = {
                        'title': {
                            self.language_code: document_title
                        },
                        'md5': document_hash,
                        'releaseDate': date.strftime('%d/%m/%Y'),
                        'url': document_url
                    }
                    json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                existed_docs.append(document_hash)
            except FileExistsError:
                print("\tDirectory path already exists, continue.")
        return existed_docs


    def get_docs_guidance_1(self, existing_docs=[], overwrite=False, to_print=True):
        existed_docs = []
        source = {
            "host": "https://www.dvi.gov.lv",
            "start_path": "/lv/dvi"
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        if to_print:
            print('Page:\t', page_url)
        page_source = self.get_source(page_url=page_url)
        results_soup = BeautifulSoup(page_source.text, 'html.parser')
        assert results_soup
        block_content = results_soup.find('div', class_='block-ministry-content')
        node = block_content.find('div', class_='node')
        content = node.find('div', class_='content')
        for paragraph in content.find_all('div', class_='paragraph--view-mode--default'):
            card_body = paragraph.find('div', class_='card-body')
            assert card_body
            file_links = card_body.find('div', class_='file-links')
            assert file_links
            icon_exclamation = file_links.find('span', class_='icon-exclamation')
            assert icon_exclamation
            date_str = icon_exclamation.get('title').split()[-1].rstrip('</p>')
            print('date: ', date_str)
            tmp = datetime.datetime.strptime(date_str, '%d.%m.%Y')
            date = datetime.date(tmp.year, tmp.month, tmp.day)
            if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                continue
            a = file_links.find('a')
            document_href = a.get('href')
            document_url = host + document_href
            document_title = a.get_text()
            print('document_title: ', document_title)
            print('\tdocument_url: ', document_url)
            document_hash = hashlib.md5(document_title.encode()).hexdigest()
            if document_hash in existing_docs and overwrite == False:
                if to_print:
                    print('\tSkipping existing document:\t', document_hash)
                continue
            if to_print:
                print("\tDocument:\t", document_hash)
            document_response = None
            try:
                document_response = requests.request('GET', document_url)
                document_response.raise_for_status()
            except requests.exceptions.HTTPError as error:
                pass
            if document_response is None:
                continue
            document_content = document_response.content
            dpa_folder = self.path
            document_folder = dpa_folder + '/' + 'Guidance' + '/' + document_hash
            # document_folder = dpa_folder + '/latvia' + '/' + 'Guidance' + '/' + document_hash
            try:
                os.makedirs(document_folder)

                with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                    f.write(document_content)
                with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                    document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                    f.write(document_text)
                with open(document_folder + '/' + 'metadata.json', 'w') as f:
                    metadata = {
                        'title': {
                            self.language_code: document_title
                        },
                        'md5': document_hash,
                        'releaseDate': date.strftime('%d/%m/%Y'),
                        'url': document_url
                    }
                    json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                existed_docs.append(document_hash)
            except FileExistsError:
                print("Directory path already exists, continue.")
        return existed_docs

    # this method only scrape pdf files
    def get_docs_guidance_2(self, existing_docs=[], overwrite=False, to_print=True):
        existed_docs = []
        source = {
            "host": "https://www.dvi.gov.lv",
            "start_path": "/lv/edak-pamatnostadnes"
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        if to_print:
            print('Page:\t', page_url)
        page_source = self.get_source(page_url=page_url)
        results_soup = BeautifulSoup(page_source.text, 'html.parser')
        assert results_soup
        block_content = results_soup.find('div', class_='block-ministry-content')
        node = block_content.find('div', class_='node')
        content = node.find('div', class_='content')
        for paragraph in content.find_all('div', class_='paragraph--view-mode--default'):
            file_links = paragraph.find('div', class_='file-links')
            if file_links is None:
                continue
            icon_exclamation = file_links.find('span', class_='icon-exclamation')
            assert icon_exclamation
            date_str = icon_exclamation.get('title').split()[-1].rstrip('</p>')
            print('date: ', date_str)
            tmp = datetime.datetime.strptime(date_str, '%d.%m.%Y')
            date = datetime.date(tmp.year, tmp.month, tmp.day)
            if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                continue
            a = file_links.find('a')
            document_href = a.get('href')
            document_url = host + document_href
            document_title = a.get_text()
            print('document_title: ', document_title)
            print('\tdocument_url: ', document_url)
            document_hash = hashlib.md5(document_title.encode()).hexdigest()
            if document_hash in existing_docs and overwrite == False:
                if to_print:
                    print('\tSkipping existing document:\t', document_hash)
                continue
            if to_print:
                print("\tDocument:\t", document_hash)
            document_response = None
            try:
                document_response = requests.request('GET', document_url)
                document_response.raise_for_status()
            except requests.exceptions.HTTPError as error:
                pass
            if document_response is None:
                continue
            document_content = document_response.content
            dpa_folder = self.path
            document_folder = dpa_folder + '/' + 'Guidance_2' + '/' + document_hash
            # document_folder = dpa_folder + '/latvia' + '/' + 'Guidance_2' + '/' + document_hash
            try:
                os.makedirs(document_folder)

                with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                    f.write(document_content)
                with open(document_folder + '/' + self.language_code + '.txt', 'wb') as f:
                    document_text = textract.process(document_folder + '/' + self.language_code + '.pdf')
                    f.write(document_text)
                with open(document_folder + '/' + 'metadata.json', 'w') as f:
                    metadata = {
                        'title': {
                            self.language_code: document_title
                        },
                        'md5': document_hash,
                        'releaseDate': date.strftime('%d/%m/%Y'),
                        'url': document_url
                    }
                    json.dump(metadata, f, indent=4, sort_keys=True, ensure_ascii=False)
                existed_docs.append(document_hash)
            except FileExistsError:
                print("Directory path already exists, continue.")
        return existed_docs


import os
import json
import math
import nltk
from nltk.tokenize import word_tokenize
from nltk.stem import WordNetLemmatizer
from collections import Counter
from .preprocess_words_service import PreprocessWordsService

class DPAStopwordsService():
    def __init__(self, path):
        self.path = path
        f = open(os.path.abspath("gdpr/assets/eu-members.json"), 'r')
        self.eu_members = json.load(f)
        f.close()

    def lookup_code(self, name):
        name = name.lower().replace(' ', '-')
        code = None
        for cand_code, cand_name in self.eu_members.items():
            if name == cand_name.lower().replace(' ', '-'):
                code = cand_code.lower()
                break
        if code is None:
            raise ValueError('Country code for country with name \'%s\' not found.' % name)
        return code

    def for_code(self, country_code, n=35, to_save=True):
        # precondition on self.path
        country_code = country_code.lower()
        lemmatizer = WordNetLemmatizer()
        tmp_word_count_docs, word_occ_docs, num_docs = {}, {}, {}
        for root, dirs, files in os.walk(self.path, topdown=True):
            split = root.split('/')
            if len(split) < 5:
                continue
            if len(split) == 5 and country_code != self.lookup_code(split[-2]):
                continue
            country_code_ = self.lookup_code(split[-2])
            country_code_ = country_code_.lower()
            if country_code_ not in tmp_word_count_docs.keys():
                tmp_word_count_docs[country_code_] = []
                word_occ_docs[country_code_] = {}
                num_docs[country_code_] = 0
            text_index = -1
            try:
                text_index = files.index('en.txt')
            except:
                pass
            if text_index == -1:
                continue
            f = open(root + '/' + files[text_index], 'r')
            text = f.read()
            f.close()
            words = word_tokenize(text)
            words = PreprocessWordsService().for_words(words)
            words = [lemmatizer.lemmatize(w) for w in words]
            tmp_word_count_docs[country_code_].extend(words)
            for w in set(words):
                if w not in word_occ_docs[country_code_].keys():
                    word_occ_docs[country_code_][w] = 1
                else:
                    word_occ_docs[country_code_][w] += 1
            num_docs[country_code_] += 1
        if bool(tmp_word_count_docs) is False:
            return {}
        word_count_docs = {}
        for country_code_ in tmp_word_count_docs.keys():
            country_code_ = country_code_.lower()
            x_min, x_max = 1, num_docs[country_code_]
            norm_word_occ_docs = {country_code_: {}}
            for k, v in word_occ_docs[country_code_].items():
                norm_word_occ_docs[country_code_][k] = (v - x_min) / (x_max - x_min)
            word_count_docs[country_code_] = {}
            for k, v in Counter(tmp_word_count_docs[country_code_]).items():
                word_count_docs[country_code_][k] = math.floor(
                    v * norm_word_occ_docs[country_code_][k]
                )
        stopwords = {}
        for k,v in word_count_docs.items():
            dpa_counter = Counter(v)
            dpa_counter += Counter()
            stopwords[k] = dict(dpa_counter.most_common(n))
            """pareto_princ = 0.8
            n = num_docs[k] - (math.ceil(num_docs[k] * (1.0-pareto_princ)))
            if n == 0:
                stopwords[k] = dict(Counter(v).most_common(5))
            else:
                #for w, freq in Counter(v).items():
                    #if freq >= n:
                    #    stopwords[k][w] = freq
                for w in sorted(dpa_counter, key=dpa_counter.get, reverse=True):
                    freq = dpa_counter[w]
                    if freq < n:
                        break
                    stopwords[k][w] = freq"""
        if to_save:
            filename = "dpa-stopwords.json"
            f = open(os.path.abspath("gdpr/assets/" + filename), 'r')
            dpa_stopwords = json.load(f)
            f.close()
            dpa_stopwords.update(stopwords)
            with open(os.path.abspath("gdpr/assets/" + filename), 'w') as outfile:
                json.dump(dpa_stopwords, outfile, indent=4, sort_keys=False)
        return stopwords[country_code]



class TranslateFilePolicy():
    allowed_formats = ['txt']
    def is_allowed(self, filename):
        split = filename.split('.')
        format_ = split[1]
        return (format_ in self.allowed_formats)



import os
import math
import requests
import json
import datetime
import hashlib
import dateparser
import re
import csv
from pygdpr.models.dpa import DPA
from bs4 import BeautifulSoup
from pygdpr.services.filename_from_path_service import filename_from_path_service
from pygdpr.services.pdf_to_text_service import PDFToTextService
from pygdpr.specifications import pdf_file_extension_specification
from pygdpr.specifications.should_retain_document_specification import ShouldRetainDocumentSpecification
from pygdpr.models.common.pagination import Pagination
from pygdpr.policies.gdpr_policy import GDPRPolicy
import textract
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from pygdpr.models.dpa.germany.dpa.federal_DPA import FederalDPA
from pygdpr.models.dpa.germany.dpa.baden_wurttemberg import BadenWurttemberg
from pygdpr.models.dpa.germany.dpa.bavaria_privatesector import Bavaria_PrivateSector
from pygdpr.models.dpa.germany.dpa.bavaria_publicsector import Bavaria_PublicSector
from pygdpr.models.dpa.germany.dpa.berlin import Berlin
from pygdpr.models.dpa.germany.dpa.brandenburg import Brandenburg
from pygdpr.models.dpa.germany.dpa.bremen import Bremen
from pygdpr.models.dpa.germany.dpa.hamburg import Hamburg
from pygdpr.models.dpa.germany.dpa.hessen import Hessen
from pygdpr.models.dpa.germany.dpa.mecklenburg_vorpommern import MecklenburgVorpommern
from pygdpr.models.dpa.germany.dpa.lower_saxony import LowerSaxony
from pygdpr.models.dpa.germany.dpa.rhineland_palatinate import RhinelandPalatinate
from pygdpr.models.dpa.germany.dpa.saarland import Saarland
from pygdpr.models.dpa.germany.dpa.saxony import Saxony
from pygdpr.models.dpa.germany.dpa.saxony_anhalt import SaxonyAnhalt
from pygdpr.models.dpa.germany.dpa.schleswig_holstein import SchleswigHolstein
from pygdpr.models.dpa.germany.dpa.thuringia import Thuringia

#from pygdpr.pygdpr.models.dpa.germany.dpa.federal_DPA import FederalDPA
#from pygdpr.pygdpr.models.dpa.germany.dpa.federal_DPA import FederalDPA
#from pygdpr.pygdpr.models.dpa.germany.dpa.federal_DPA import FederalDPA


class Germany(DPA):
    def __init__(self, path=os.curdir):
        country_code='de'
        super().__init__(country_code, path)

    def get_docs(self, existing_docs=[], overwrite=False, to_print=True):
        '''
        print('docs - dpa germany')
        added_docs = []
        added_docs += FederalDPA().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += BadenWurttemberg().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += Bavaria_PrivateSector().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += Bavaria_PublicSector().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += Berlin().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += Brandenburg().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += Bremen().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += Hamburg().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += Hessen().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += MecklenburgVorpommern().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += LowerSaxony().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += RhinelandPalatinate().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += Saarland().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += Saxony().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += SaxonyAnhalt().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += SchleswigHolstein().get_docs(overwrite=False, to_print=True, path=self.path)
        #added_docs += Thuringia().get_docs(overwrite=False, to_print=True, path=self.path)
        return added_docs
'''


from pygdpr.services.metadata_service import MetadataService
from pygdpr.services.list_eu_currencies_service import ListEUCurrenciesService
from price_parser import Price
import nltk

class MonetaryPenaltiesMetadataService(MetadataService):
    def join_numeric_words(self, words):
        ws = words
        joined_ws = []
        i = 0
        while i < len(ws)-1:
            if ws[i].isnumeric() is False:
                joined_ws.append(ws[i])
            else:
                j = i+1
                while j < len(ws) and ws[j].isnumeric():
                    j+=1
                joined_ws.append("".join(ws[i:j]))
                del ws[i+1:j]
            i+=1
        return joined_ws

    def for_text(self, text):
        monetary_penalties = []
        language_code = 'en'
        for s in nltk.sent_tokenize(text):
            words = nltk.word_tokenize(s)
            words = [w.lower() for w in words]
            words = self.join_numeric_words(words)
            if 'penalty' not in words and 'fine' not in words: # stem the words. - what about plurals?
                continue
            if len(words) < 2:
                continue
            subwords = []
            currency_symbols = []
            for i in range(len(words)):
                w = words[i]
                list_eu_currencies = ListEUCurrenciesService()
                eu_currencies = list_eu_currencies.get()
                for c in eu_currencies:
                    for p in c['plural']:
                        if w == p and words[i-1].isnumeric():
                            currency_symbols.append(c['symbol'])
                            subwords.append(words[i-1] + ' ' + w)
                    if w == c['code'].lower() and words[i+1].isnumeric():
                        currency_symbols.append(c['symbol'])
                        subwords.append(w + ' ' + words[i+1])
                    if c['symbol'] is None:
                        continue
                    if w.startswith(c['symbol']) or w.endswith(c['symbol']):
                        currency_symbols.append(c['symbol'])
                        subwords.append(words[i])
                    elif w == c['symbol'] and w[i+1].isnumeric():
                        currency_symbols.append(c['symbol'])
                        subwords.append(words[i] + ' ' + words[i+1])
                    elif w == c['symbol'] and w[i-1].isnumeric():
                        currency_symbols.append(c['symbol'])
                        subwords.append(words[i-1] + ' ' + words[i])
            assert len(subwords) == len(currency_symbols)
            for i in range(len(subwords)):
                currency_symbol = currency_symbols[i]
                price = Price.fromstring(subwords[i], currency_hint=currency_symbol)
                if currency_symbol is None:
                    continue
                if price.amount_float is None:
                    continue
                filtered_currencies = list(filter(lambda c: c["symbol"] == currency_symbol, eu_currencies))
                if len(filtered_currencies) == 0:
                    continue
                currency = filtered_currencies[0]
                amount = price.amount_float
                monetary_penalty = {
                    'amount': amount,
                    'currency_code': currency['code'],
                    'language_code': language_code,
                    'text': s
                }
                monetary_penalties.append(monetary_penalty)
        return monetary_penalties



class GDPRChapterSectionService():
    def __init__(self):
        self.comps_specs = [
            # (chapter, [(section:nullable, [min_art, max_art])])
            (1, [(None, [1,4])]),
            (2, [(None, [5,11])]),
            (3, [(1, [12,12]), (2, [13,15]), (3, [16,20]), (4, [21,22]), (5, [23,23])]),
            (4, [(1, [24,31]), (2, [32,34]), (3, [35,36]), (4, [37,39]), (5, [40,43])]),
            (5, [(None, [44, 50])]),
            (6, [(1, [51, 54]), (2, [55, 59])]),
            (7, [(1, [60, 62]), (2, [63, 67]), (3, [68, 76])]),
            (8, [(None, [77, 84])]),
            (9, [(None, [85, 91])]),
            (10, [(None, [92, 93])]),
            (11, [(None, [94, 99])])
        ]

    def for_article(self, article):
        chapter, section = None, None
        for cand_chapter, section_list in self.comps_specs:
            for cand_section, art_limit in section_list:
                min_art, max_art = art_limit[0], art_limit[1]
                if min_art <= article <= max_art:
                    chapter = cand_chapter
                    section = cand_section
                    break
        if chapter == None:
            raise AttributeError('Unable to infer chapter for value of article: ' + str(article))
        return (chapter, section)



import datetime
from pygdpr.policies.gdpr_policy import GDPRPolicy

class ShouldRetainDocumentSpecification():
    def is_satisfied_by(self, cand): #cand = date
        today = datetime.date.today()
        margin = today - GDPRPolicy().implementation_date() # timedelta
        return today - margin <= cand <= today + margin



import os
import shutil
import math
import requests
import json
import datetime
import hashlib
import textract
from pygdpr.models.dpa import DPA
from bs4 import BeautifulSoup
import dateparser
from pygdpr.services.filename_from_path_service import filename_from_path_service
from pygdpr.specifications import pdf_file_extension_specification
from pygdpr.specifications.should_retain_document_specification import ShouldRetainDocumentSpecification
from pygdpr.models.common.pagination import Pagination
from pygdpr.policies.gdpr_policy import GDPRPolicy
from pygdpr.services.pdf_to_text_service import PDFToTextService
import docx2txt
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from pygdpr.policies.webdriver_exec_policy import WebdriverExecPolicy
from selenium.webdriver.common.keys import Keys
import time


class UnitedKingdom(DPA):
    def __init__(self, path=os.curdir):
        country_code='GB'
        super().__init__(country_code, path)

    def update_pagination(self, pagination=None, page_soup=None, driver=None, start_path="Reports"):
        source = {
            "host": "https://ico.org.uk",
            "start_path_Reports": "/action-weve-taken/audits-and-overview-reports/?facet_type=&facet_sector=&facet_date=custom&date_from=01%2F05%2F2018&date_to=",
            "start_path_Enforcements": "/action-weve-taken/enforcement/"
        }
        host = source['host']
        if start_path != "Reports":
            start_path = source['start_path_Enforcements']
        else:
            start_path = source['start_path_Reports']
        if pagination is None:
            pagination = Pagination()
            pagination.add_item(host + start_path)
        else:
            pagination = Pagination()

            button_next = page_soup.find('nav', class_='article-navigation')
            if button_next is None:
                return pagination
            page_link = button_next.find('a', class_='button button-top').get('href')
            if page_link is None:
                return pagination
            print("page_link:", page_link)
            pagination.add_item(host + page_link)
            print('added link to pagination: ', host + page_link)

        return pagination


    def get_source(self, page_url=None, driver=None):
        assert (page_url is not None)
        results_response = None
        try:
            results_response = requests.request('GET', page_url)
            results_response.raise_for_status()
        except requests.exceptions.HTTPError as error:
            pass
        return results_response

    def get_docs(self, existing_docs=[], overwrite=False, to_print=True):
        added_docs = []
        # call all the get_docs_X() functions
        added_docs += self.get_docs_Notices(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Enforcements(existing_docs=[], overwrite=False, to_print=True)
        added_docs += self.get_docs_Reports(existing_docs=[], overwrite=False, to_print=True)
        return added_docs


    def get_docs_Notices(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== United Kingdom Notices =========================")
        iteration = 1
        existed_docs = []
        existed_date = []
        source = {
            "host": "https://icosearch.ico.org.uk",
            "start_path": "/s/search.html?collection=ico-meta&profile=decisions&query&query=GDPR"
        }
        host = source['host']
        start_path = source['start_path']
        page_url = host + start_path
        page_source = self.get_source(page_url=page_url)
        if page_source is None:
            print("This url is not exist.")
        results_soup = BeautifulSoup(page_source.text, 'html.parser')
        assert results_soup
        maincolumn = results_soup.find('div', class_='maincolumn')
        assert maincolumn
        resultlist = maincolumn.find('div', class_='resultlist')
        assert resultlist
        # s1. Results
        for itemlink in resultlist.find_all('div', class_='itemlink'):
            print('\n------------ Document ' + str(iteration) + ' ------------')
            iteration += 1
            time.sleep(2)
            result_link = itemlink.find('a')
            assert result_link
            text_small = itemlink.find('p', class_='text-small')
            assert text_small
            date_str = text_small.get_text().split(',')[0].strip()
            tmp = dateparser.parse(date_str, languages=[self.language_code]) # datetime.datetime.strptime(date_str, '%d %B %Y')
            date = datetime.date(tmp.year, tmp.month, tmp.day)
            print('\tdate:\t', str(date))

            uk_left_eu = datetime.date(2020, 1, 31)
            # check whether this article release after UK left EU
            #if date.year > uk_left_eu.year or (date.month > uk_left_eu.month and date.year == uk_left_eu.year):
            #    print("This article release after UK left EU.")
            #    continue

            if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                continue
            # s2. Documents
            h2 = result_link.find('h2', class_='h3')
            assert h2
            document_title = h2.get_text().strip()
            print('\tdocument_title:\t', document_title)
            document_hash = hashlib.md5(document_title.encode()).hexdigest()

            # check whether this articles already exist
            if document_hash in existed_docs and date in existed_date:
                if to_print:
                    print('\tSkipping existing document:\t', document_hash)
                continue

            document_href = result_link.get('title') # result_link.get('href')
            assert document_href
            if document_href.endswith('.pdf') is False:
                print("found a document which is not of mimeType PDF.")
                continue
            host = "https://icosearch.ico.org.uk" # "https://ico.org.uk"
            document_url = document_href# host + document_href
            print('\tdocument_url:\t', document_url)
            if to_print:
                print("\tDocument:\t", document_hash)
            document_response = None
            try:
                document_response = requests.request('GET', document_url)
                document_response.raise_for_status()
            except requests.exceptions.HTTPError as error:
                pass
            if document_response is None:
                continue

            dpa_folder = self.path
            if document_hash in existed_docs:
                document_folder = dpa_folder + '/' + 'Notices' + '/' + document_hash + ' -02'
            else:
                document_folder = dpa_folder + '/' + 'Notices' + '/' + document_hash
            try:
                os.makedirs(document_folder)
                file_url = document_url

                try:
                    file_response = requests.request('GET', file_url)
                    file_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    if to_print:
                        print(error)
                    pass
                if file_response is None:
                    continue
                file_content = file_response.content
                with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                    f.write(file_content)
                    with open(document_folder + '/' + self.language_code + '.txt', 'w') as f:
                        document_text = PDFToTextService().text_from_pdf_path(document_folder + '/' + self.language_code + '.pdf')
                        f.write(document_text)

                with open(document_folder + '/' + 'metadata.json', 'w') as f:
                    metadata = {
                        'title': {
                            self.language_code: document_title
                        },
                        'md5': document_hash,
                        'releaseDate': date.strftime('%d/%m/%Y'),
                        'url': document_url
                    }
                    json.dump(metadata, f, indent=4, sort_keys=True)
                existed_docs.append(document_hash)
                existed_date.append(date)
                print('\n')
            except FileExistsError:
                print("\tDirectory path already exists, continue.")

        return existed_docs

    def get_docs_Reports(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== United Kingdom Reports =========================")
        iteration = 1
        existed_docs = []
        hashcode_with_date = {}
        pagination = self.update_pagination()
        # s0. Pagination
        while pagination.has_next():
            page_url = pagination.get_next()
            if to_print:
                print('Page:\t', page_url)
            page_source = self.get_source(page_url=page_url)
            if page_source is None:
                print("None page source")
                continue
            results_soup = BeautifulSoup(page_source.text, 'html.parser')
            assert results_soup

            resultlist = results_soup.find('div', class_='resultlist')
            assert resultlist
            # s1. Results
            for itemlink in resultlist.find_all('div', class_='itemlink'):

                print('\n------------ Document ' + str(iteration) + ' ------------')
                iteration += 1
                # time.sleep(2)
                result_link = itemlink.find('a')
                assert result_link
                text_small = itemlink.find('p', class_='text-small')
                assert text_small
                date_str = text_small.get_text().split(',')[0].strip()
                tmp = dateparser.parse(date_str, languages=[self.language_code]) # datetime.datetime.strptime(date_str, '%d %B %Y')
                date = datetime.date(tmp.year, tmp.month, tmp.day)

                # addition: we can check whether this article release after UK left EU
                # uk_left_eu = datetime.date(2020, 1, 31)
                # check whether this article release after UK left EU
                #if date.year > uk_left_eu.year or (date.month > uk_left_eu.month and date.year == uk_left_eu.year):
                #    print("This article release after UK left EU.")
                #    continue

                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    continue
                # s2. Documents
                h2 = result_link.find('h2', class_='h3')
                assert h2
                document_title = h2.get_text().strip()

                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                # check whether this articles already exist
                if document_hash in hashcode_with_date.keys() and hashcode_with_date[document_hash] == date:
                    if to_print:
                        print('\tAlready exist, skip:\t', document_hash)
                    continue

                print('\tdocument_title:\t', document_title)
                print('\tdate:\t', date)
                document_href = result_link.get('href')
                assert document_href

                host = "https://ico.org.uk"
                document_url = host + document_href
                print('\tdocument_url:\t', document_url)
                if to_print:
                    print("\tDocument:\t", document_hash)
                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    pass
                if document_response is None:
                    continue

                dpa_folder = self.path
                if document_hash in existed_docs:
                    document_folder = dpa_folder + '/' + 'Reports' + '/' + document_hash + ' -02'
                else:
                    document_folder = dpa_folder + '/' + 'Reports' + '/' + document_hash
                try:
                    os.makedirs(document_folder)

                    document_soup = BeautifulSoup(document_response.text, 'html.parser')
                    assert document_soup

                    article_content = document_soup.find_all('div', class_='article-content')
                    assert article_content
                    document_text = ''
                    for i in article_content:
                        document_text += i.get_text()
                    with open(document_folder + '/' + self.language_code + '-' + 'article_content' + '.txt', 'w') as f:
                        f.write(document_text)

                    aside_further = document_soup.find('aside', class_='aside-further')
                    if aside_further is not None:
                        count_articles = 0
                        for articles in aside_further.find_all('li'):
                            title = articles.find('h3').get_text()
                            print("\t\tfile_title: ", title)
                            article_url = articles.find('a').get('href')

                            if article_url is not None:
                                file_url = host + article_url
                                print("\t\tfile_url: ", file_url)
                                try:
                                    file_response = requests.request('GET', file_url)
                                    file_response.raise_for_status()
                                except requests.exceptions.HTTPError as error:
                                    if to_print:
                                        print(error)
                                    pass
                                if file_response is None:
                                    continue
                                count_articles += 1
                                file_content = file_response.content
                                if count_articles > 1:
                                    with open(document_folder + '/' + self.language_code + '-' + str(count_articles) + '.pdf', 'wb') as f:
                                        f.write(file_content)
                                    with open(document_folder + '/' + self.language_code + '_' + str(count_articles) + '.txt', 'w') as f:
                                        document_text = PDFToTextService().text_from_pdf_path(document_folder + '/' + self.language_code + '-' + str(count_articles) + '.pdf')
                                        f.write(document_text)
                                else:
                                    with open(document_folder + '/' + self.language_code + '.pdf', 'wb') as f:
                                        f.write(file_content)
                                    with open(document_folder + '/' + self.language_code + '.txt', 'w') as f:
                                        document_text = PDFToTextService().text_from_pdf_path(document_folder + '/' + self.language_code + '.pdf')
                                        f.write(document_text)

                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True)
                    existed_docs.append(document_hash)
                    hashcode_with_date[document_hash] = date
                    print('\n')
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
            # s0. Pagination
            pagination = self.update_pagination(pagination=pagination, page_soup=results_soup)
        return existed_docs

    def get_docs_Enforcements(self, existing_docs=[], overwrite=False, to_print=True):
        print("\n======================== United Kingdom Enforcements=========================")
        existed_docs = []
        existed_dates = []
        hashcode_with_type = ""
        hashcode_dict = {}
        pagination = self.update_pagination(start_path="Enforcements")
        # s0. Pagination
        while pagination.has_next():
            page_url = pagination.get_next()
            if to_print:
                print('Page:\t', page_url)
            page_source = self.get_source(page_url=page_url)
            if page_source is None:
                print("None page source")
                continue
            time.sleep(2)

            results_soup = BeautifulSoup(page_source.text, 'html.parser')
            assert results_soup

            exec_path = WebdriverExecPolicy().get_system_path()
            options = webdriver.ChromeOptions()
            options.add_argument('headless')
            driver_doc = webdriver.Chrome(options=options, executable_path=exec_path)
            driver_doc.get(page_url)
            resultlist = driver_doc.find_element_by_class_name('resultlist')

            # s1. Results
            for itemlink in resultlist.find_elements_by_class_name('itemlink'):
                time.sleep(2)
                result_link = itemlink.find_element_by_tag_name('a')
                assert result_link
                text_small = itemlink.find_element_by_class_name('text-small')
                assert text_small
                date_str = text_small.text.split(',')[0].strip()
                notice_type = text_small.text.split(',')[1].strip()
                tmp = dateparser.parse(date_str, languages=[self.language_code])
                date = datetime.date(tmp.year, tmp.month, tmp.day)

                if ShouldRetainDocumentSpecification().is_satisfied_by(date) is False:
                    continue
                # s2. Documents
                h2 = result_link.find_element_by_class_name('h3')
                assert h2
                document_title = h2.text.strip()

                document_hash = hashlib.md5(document_title.encode()).hexdigest()
                hashcode_with_type = document_hash + '-' + notice_type

                # check whether this articles already exist
                if hashcode_with_type in hashcode_dict.keys() and hashcode_dict[hashcode_with_type] == date:
                    if to_print:
                        print('\tAlready exist, skip:\t', document_hash)
                    continue

                print('document_title:\t', document_title)
                print('\tdate:\t', date)
                document_href = result_link.get_attribute('href')
                assert document_href
                host = "https://ico.org.uk"
                document_url = host + document_href
                print('\tdocument_url:\t', document_url)
                if to_print:
                    print("\tDocument:\t", document_hash)
                document_response = None
                try:
                    document_response = requests.request('GET', document_url)
                    document_response.raise_for_status()
                except requests.exceptions.HTTPError as error:
                    pass
                except requests.exceptions.ConnectionError as error:
                    if to_print:
                        print("\tError:", error)
                    pass
                if document_response is None:
                    continue

                dpa_folder = self.path
                if document_hash in existed_docs:
                    document_folder = dpa_folder + '/' + 'Enforcements' + '/' + document_hash + ' -02'
                else:
                    document_folder = dpa_folder + '/' + 'Enforcements' + '/' + document_hash
                try:
                    os.makedirs(document_folder)

                    document_soup = BeautifulSoup(document_response.text, 'html.parser')
                    assert document_soup
                    article_content = document_soup.find_all('div', class_='article-content')
                    if article_content is not None:
                        document_text = ''
                        for i in article_content:
                            document_text += i.get_text()
                        with open(document_folder + '/' + self.language_code + '-' + 'article_content' + '.txt', 'w') as f:
                            f.write(document_text)
                    aside_further = document_soup.find('aside', class_='aside-further')
                    if aside_further is not None:
                        #count_articles = 0
                        count = 0
                        for articles in aside_further.find_all('li'):
                            title = articles.find('h3').get_text()
                            print("\t\tfile_title: ", title)
                            article_url = articles.find('a').get('href')
                            if article_url is not None:
                                file_url = host + article_url
                                print("\t\tfile_url: ", file_url)
                                file_response = None
                                try:
                                    file_response = requests.request('GET', file_url)
                                    file_response.raise_for_status()
                                except requests.exceptions.HTTPError as error:
                                    if to_print:
                                        print(error)
                                    pass
                                except requests.exceptions.ConnectionError as error:
                                    if to_print:
                                        print("\tError:",error)
                                    pass
                                if file_response is None:
                                    continue
                                # the current notice pdf page
                                if file_url.endswith('.pdf'):
                                    count += 1
                                    if count == 2:
                                        print("This is pdf file with another notice type")
                                        continue
                                    file_content = file_response.content
                                    with open(document_folder + '/' + self.language_code + '-' + notice_type + '.pdf',
                                            'wb') as f:
                                        f.write(file_content)
                                    with open(document_folder + '/' + self.language_code + '-' + notice_type + '.txt',
                                            'wb') as f:
                                        document_text = textract.process(
                                            document_folder + '/' + self.language_code + '-' + notice_type + '.pdf')
                                        f.write(document_text)
                                elif file_url.endswith('.docx'):
                                    # set a download path. Use selenium to download it
                                    exec_path = WebdriverExecPolicy().get_system_path()
                                    options = webdriver.ChromeOptions()
                                    prefs = {"download.default_directory": document_folder}
                                    options.add_argument('headless')
                                    options.add_experimental_option("prefs", prefs)
                                    driver_doc = webdriver.Chrome(options=options, executable_path=exec_path)
                                    # open google search, and put the document_url into search box, and press enter

                                    driver_doc.get(document_url)
                                    button = driver_doc.find_element_by_xpath('//*[@id="startcontent"]/article/div[2]/div/aside/ul/li[1]/a')
                                    button.click()
                                    time.sleep(5)
                                    files = os.listdir(document_folder)
                                    for index, file in enumerate(files):
                                        if 'penalty' in os.path.basename(file):
                                            os.rename(os.path.join(document_folder, file), os.path.join(document_folder, self.language_code + '-' + 'final-penalty' + '.docx'))
                                        document_content = docx2txt.process(document_folder + '/' + self.language_code + '-' + 'final-penalty' + '.docx')
                                        with open(
                                                document_folder + '/' + self.language_code + '-' + 'final-penalty' + '.txt','w') as f:
                                            f.write(document_content)
                                else:
                                    file_soup = BeautifulSoup(file_response.text, 'html.parser')
                                    assert file_soup
                                    maincolumn = file_soup.find('div', class_='maincolumn column column-8')
                                    aside_further = maincolumn.find('aside', class_='aside-further')
                                    # only has the news
                                    if aside_further is None:
                                        file_content = maincolumn.find('div', class_='article-content')
                                        file_text = file_content.get_text()
                                        with open(
                                                document_folder + '/' + self.language_code + '-' + 'news' + '.txt',
                                                'w') as f:
                                            f.write(file_text)
                                    else:
                                        continue

                    with open(document_folder + '/' + 'metadata.json', 'w') as f:
                        metadata = {
                            'title': {
                                self.language_code: document_title
                            },
                            'md5': document_hash,
                            'releaseDate': date.strftime('%d/%m/%Y'),
                            'url': document_url
                        }
                        json.dump(metadata, f, indent=4, sort_keys=True)
                    existed_docs.append(document_hash)
                    existed_dates.append(date)
                    hashcode_dict[hashcode_with_type] = date
                    print('\n')
                except FileExistsError:
                    print("\tDirectory path already exists, continue.")
            # s0. Pagination
            pagination = self.update_pagination(pagination=pagination, page_soup=results_soup)
        return existed_docs



import os
import json

class ValidGDPRArticleRefSpecification():
    def is_satisfied_by(self, cand): # cand signature: (art, par, let)
        art, par, let = cand
        art = str(art) if art is not None else None
        par = str(par) if par is not None else None
        if art is None:
            return False
        if art.isdigit() == False:
            return False
        if par is not None and par.isdigit() == False:
            return False
        if let is not None and let.isalpha() == False:
            return False
        path = 'gdpr/assets/gdpr-article-validation-tree.json'
        f = open(os.path.abspath(path), 'r')
        validation_tree = json.load(f)
        f.close()
        if art not in validation_tree.keys():
            return False
        # assume: art exists in validation tree
        if par is not None:
            if let is None:
                return (par in validation_tree[art].keys())
            else:
                return (
                    par in validation_tree[art].keys() and\
                    let in validation_tree[art][par].keys()
                )
        # assume: par is None (null)
        if let is not None:
            return (let in validation_tree[art].keys())
        # assume: both par and let is None (null)
        return True



from pygdpr.services.metadata_service import *
from pygdpr.specifications.absolute_date_specification import *
import dateparser
from dateparser.search import search_dates
import datetime
import nltk

class TimelineMetadataService(MetadataService):
    def for_text(self, text):
        timeline = []
        language_code = 'en'
        date_formats = ['%d-%m-%Y', '%d %B %Y']
        dateparser.date._DateLocaleParser._try_freshness_parser = lambda self: False
        sents = nltk.sent_tokenize(text)
        for s in sents:
            words = nltk.word_tokenize(s)
            words = [w.lower() for w in words]
            words = [w for w in words if w.isdigit() or w.isalpha()]
            try:
                matches = search_dates(s, languages=['en'], settings={
                    'STRICT_PARSING': True,
                    'PREFER_DATES_FROM': 'past'
                })
                if matches is None:
                    continue
                if len(matches) == 0:
                    continue
                for m in matches:
                    if AbsoluteDateSpecification().is_satisfied_by(m) is False:
                        continue
                    date = m[1]
                    date_str = datetime.datetime.strftime(date, '%d/%m/%Y')
                    event = {
                        'date': date_str,
                        'language_code': language_code,
                        'text': s
                    }
                    timeline.append(event)
            except:
                pass
        return timeline



import nltk
import datetime
class AbsoluteDateSpecification():
    def day_is_present(self, words, date):
        return (
            date.strftime('%d') in words or\
            date.strftime('%-d') in words
        )
    def month_is_present(self, words, date):
        return (
            date.strftime('%b').lower() in words or\
            date.strftime('%B').lower() in words or\
            date.strftime('%m').lower() in words or\
            date.strftime('%-m').lower() in words
        )
    def year_is_present(self, words, date):
        return (
            date.strftime('%y') in words or\
            date.strftime('%Y') in words
        )
    def date_tokenize(self, words):
        tmp = []
        delimiters = ['-', '/']
        for w in words:
            dash_split = w.split('-')
            slash_split = w.split('/')
            if len(dash_split) > 1:
                tmp.extend(dash_split)
            elif len(slash_split) > 1:
                tmp.extend(slash_split)
            else:
                tmp.append(w)
        return tmp
    def is_satisfied_by(self, cand): # cand signature: (str, datetime)
        snippet, date = cand
        ws = nltk.word_tokenize(snippet)
        ws = [w.lower() for w in ws]
        ws = self.date_tokenize(ws)
        if len(ws) < 3:
            return False
        return (
            self.day_is_present(ws, date) and\
            self.month_is_present(ws, date) and\
            self.year_is_present(ws, date)
        )



from pygdpr.models.common.eu_member import EUMember
from pygdpr.models.dpa.austria import Austria
from pygdpr.models.dpa.belgium import Belgium
from pygdpr.models.dpa.bulgaria import Bulgaria
from pygdpr.models.dpa.croatia import Croatia
from pygdpr.models.dpa.cyprus import Cyprus
from pygdpr.models.dpa.czech_republic import CzechRepublic
from pygdpr.models.dpa.denmark import Denmark
from pygdpr.models.dpa.estonia import Estonia
from pygdpr.models.dpa.finland import Finland
from pygdpr.models.dpa.france import France
from pygdpr.models.dpa.germany import Germany
from pygdpr.models.dpa.greece import Greece
from pygdpr.models.dpa.hungary import Hungary
from pygdpr.models.dpa.ireland import Ireland
from pygdpr.models.dpa.italy import Italy
from pygdpr.models.dpa.latvia import Latvia
from pygdpr.models.dpa.lithuania import Lithuania
from pygdpr.models.dpa.luxembourg import Luxembourg
from pygdpr.models.dpa.malta import Malta
from pygdpr.models.dpa.netherlands import Netherlands
from pygdpr.models.dpa.poland import Poland
from pygdpr.models.dpa.portugal import Portugal
from pygdpr.models.dpa.romania import Romania
from pygdpr.models.dpa.slovakia import Slovakia
from pygdpr.models.dpa.slovenia import Slovenia
from pygdpr.models.dpa.spain import Spain
from pygdpr.models.dpa.sweden import Sweden
from pygdpr.models.dpa.united_kingdom import UnitedKingdom
import numpy as np
import os
import nltk
from nltk.corpus import stopwords
from nltk.stem import PorterStemmer
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.decomposition import PCA
from sklearn.pipeline import Pipeline
from sklearn.cluster import KMeans
from sklearn.naive_bayes import MultinomialNB
from sklearn.pipeline import make_pipeline
from imblearn.under_sampling import RandomUnderSampler
from imblearn.pipeline import make_pipeline as make_pipeline_imb
from imblearn.metrics import classification_report_imbalanced
from sklearn.model_selection import train_test_split # not sure if should keep this.
import sys
import matplotlib.pyplot as plt
from sklearn.feature_extraction.text import TfidfTransformer
from sklearn import metrics
from scipy.spatial.distance import cdist
from pygdpr.services.list_eu_members_service import ListEUMembersService
from pygdpr.policies.dpa_path_policy import DPAPathPolicy

class GDPR(object): # acts as a mediator/facade pattern :: find su:ku slides.
    """
    A class used to represent GDPR (General Data Protection Regulation).

    Attributes
    ----------
    path : str
        the path where the documents will be stored (default is os.cwd())

    Methods
    -------
    get_dpa(country_code)
        Returns an instance of a DPA corresponding to the country_code
    """
    def __init__(self, path=os.curdir):
        self.path = path

    def set_path(self):
        self.path = path

    def get_path(self):
        return self.path

    def get_dpa(self, country_code):
        """Returns an instance of a DPA corresponding to the country_code.

        Parameters
        ----------
        country_code : str
            The country code corresponding to a given country.

        Raises
        ------
        ValueError
            If country code parameter does not correspond to a valid EU Member state country.

        Returns
        -------
        DPA
            an instance of a DPA class corresponding to the country code
        """
        gdpr_path = self.get_path()
        dpa_path = DPAPathPolicy().extend_gdpr_path(gdpr_path, country_code)
        if country_code == EUMember.AUSTRIA.value:
            return Austria(dpa_path)
        elif country_code == EUMember.BELGIUM.value:
            return Belgium(dpa_path)
        elif country_code == EUMember.BULGARIA.value:
            return Bulgaria(dpa_path)
        elif country_code == EUMember.CROATIA.value:
            return Croatia(dpa_path)
        elif country_code == EUMember.CYPRUS.value:
            return Cyprus(dpa_path)
        elif country_code == EUMember.CZECH_REPUBLIC.value:
            return CzechRepublic(dpa_path)
        elif country_code == EUMember.DENMARK.value:
            return Denmark(dpa_path)
        elif country_code == EUMember.ESTONIA.value:
            return Estonia(dpa_path)
        elif country_code == EUMember.FINLAND.value:
            return Finland(dpa_path)
        elif country_code == EUMember.FRANCE.value:
            return France(dpa_path)
        elif country_code == EUMember.GREECE.value:
            return Greece(dpa_path)
        elif country_code == EUMember.HUNGARY.value:
            return Hungary(dpa_path)
        elif country_code == EUMember.IRELAND.value:
            return Ireland(dpa_path)
        elif country_code == EUMember.ITALY.value:
            return Italy(dpa_path)
        elif country_code == EUMember.LATVIA.value:
            return Latvia(dpa_path)
        elif country_code == EUMember.LITHUANIA.value:
            return Lithuania(dpa_path)
        elif country_code == EUMember.LUXEMBOURG.value:
            return Luxembourg(dpa_path)
        elif country_code == EUMember.MALTA.value:
            return Malta(dpa_path)
        elif country_code == EUMember.NETHERLANDS.value:
            return Netherlands(dpa_path)
        elif country_code == EUMember.POLAND.value:
            return Poland(dpa_path)
        elif country_code == EUMember.PORTUGAL.value:
            return Portugal(dpa_path)
        elif country_code == EUMember.ROMANIA.value:
            return Romania(dpa_path)
        elif country_code == EUMember.SLOVAKIA.value:
            return Slovakia(dpa_path)
        elif country_code == EUMember.SLOVENIA.value:
            return Slovenia(dpa_path)
        elif country_code == EUMember.SPAIN.value:
            return Spain(dpa_path)
        elif country_code == EUMember.SWEDEN.value:
            return Sweden(dpa_path)
        elif country_code == EUMember.GERMANY.value:
            return Germany(dpa_path)
        elif country_code == EUMember.UNITED_KINGDOM.value:
            return UnitedKingdom(dpa_path)
        else:
            raise ValueError(f"Could not instantiate instance of DPA for country_code: {country_code}")

    def get_docs(self, existing_docs=[], overwrite=False, to_print=True):
        added_docs = []
        list_eu_members = ListEUMembersService()
        eu_members = list_eu_members.get()
        for eu_member in eu_members:
            country_code = eu_member.value
            dpa = self.get_dpa(country_code)
            try:
                added_docs += dpa.get_docs(
                    existing_docs=existing_docs,
                    overwrite=overwrite,
                    to_print=to_print
                )
            except:
                pass
        return added_docs

    def translate_docs(self, target_languages, docs=[], overwrite=False, price_terminate_usd=0.0, quota_service=None, to_print=True):
        translated_docs = []
        list_eu_members = ListEUMembersService()
        eu_members = list_eu_members.get()
        for eu_member in eu_members:
            country_code = eu_member.value
            dpa = gdpr.get_dpa(country_code)
            translated_docs += dpa.translate_docs(
                target_languages,
                docs,
                overwrite,
                price_terminate_usd,
                quota_service,
                to_print
            )
        return translated_docs

    def classify_docs(self, docs=[], download_if_needed=True, to_print=True):
        print('Classifying docs...')
        f = open('gdpr/assets/gdpr-stopwords.txt', 'r')
        gdpr_stopwords = f.read().split('\n')
        f.close()
        stop_words = set(stopwords.words('english')).union(gdpr_stopwords)
        ps = PorterStemmer()
        tmp_X = []
        n_docs = 0
        for root, dirs, files in os.walk('../data/11-21-2019', topdown=True):
            split = root.split('/')
            if len(split) < 5:
                continue
            dochash = split[-1]
            country = split[-2]
            country = country.replace('-', ' ')
            country = country.capitalize()
            print('\tFor:', country)
            try:
                text_index = files.index('en.txt')
            except:
                continue
            f = open(root + '/' + files[text_index], 'r')
            document_text = f.read()
            f.close()
            words = nltk.word_tokenize(document_text)
            words = [w.lower() for w in words]
            words = [w for w in words if w.isalpha()]
            words = [w for w in words if w not in stop_words]
            words = [ps.stem(word) for word in words]
            # X = np.concatenate((X, np.array([[dochash, ' '.join(words)]])), axis=0)
            tmp_X.append([dochash, ' '.join(words)])
            n_docs += 1
        #if n_docs == 0 and download_if_needed:
        #    self.get_docs()
        #    self.classify_docs(docs, download_if_needed, to_print)
        X = np.array(tmp_X)
        assert n_docs > 0
        np.set_printoptions(threshold=sys.maxsize) # tmp for dev purposes.
        pipeline = Pipeline([
            ('vect', CountVectorizer()),
            ('tfidf', TfidfTransformer()),
        ])
        X_fit = pipeline.fit_transform(X[:,1]).todense()
        kmeans = KMeans(n_clusters=4, max_iter=1000, random_state=3425)
        labels = kmeans.fit_predict(X_fit)
        labels = np.column_stack((X[:,0], labels))
        if to_print:
            pca = PCA(n_components=2).fit(X_fit)
            centers2D = pca.transform(kmeans.cluster_centers_)
            data2D = pca.transform(X_fit)
            plt.plot()
            plt.title('Dataset')
            plt.scatter(data2D[:,0], data2D[:,1], c=kmeans.labels_.tolist())
            plt.scatter(centers2D[:,0], centers2D[:,1],
                        marker='x', s=200, linewidths=3, c='r')
            plt.show()
            # k means determine k
            distortions = []
            K = range(1,10)
            for k in K:
                kmeanModel = KMeans(n_clusters=k).fit(data2D) # on data2d instead?
                kmeanModel.fit(data2D)
                distortions.append(sum(np.min(cdist(data2D, kmeanModel.cluster_centers_, 'euclidean'), axis=1)) / data2D.shape[0])
            print(distortions)
            # Plot the elbow
            plt.plot(K, distortions, 'bx-')
            plt.xlabel('k')
            plt.ylabel('Distortion')
            plt.title('The Elbow Method showing the optimal k')
            plt.show()
        if len(docs) > 0:
            num_rows, num_cols = labels.shape[0], labels.shape[1]
            row_indices = []
            for i in range(num_rows):
                for j in range(num_cols):
                    if j == 0 and labels[i, j] in docs:
                        row_indices.append([i])
            assert len(row_indices) == len(docs)
            labels = labels[row_indices,:].reshape((len(docs), 2))
        return labels



import datetime
import random
import math

class DocumentKeyService():
    def __init__(self, kind, release_date, country_code):
        self.kind = kind
        self.release_date = release_date
        self.country_code = country_code

    def get_control_char(self, YYMMDD, ZZZZ):
        # omitted ambiguous letters: G, I, O, Q, Z
        pick_str = "0123456789ABCDEFHJKLMNPRSTUVWXY"
        idx = (int(YYMMDD) + int(ZZZZ)) / len(pick_str)
        idx = idx - int(idx)
        idx = idx * len(pick_str)
        idx = math.ceil(idx)
        return pick_str[idx]

    def push_key(self):
        K = self.kind[0:2].upper()
        YYMMDD = self.release_date.strftime("%y%m%d")
        ZZZZ = str(random.randint(0, 9999)).zfill(4)
        Q = self.get_control_char(YYMMDD, ZZZZ)
        C = self.country_code.upper()
        # valid separators in URL components.
        return f"{K}{YYMMDD}-{ZZZZ}{Q}+{C}"
